; Script to implement "Water Reflection" of an image
; based on dm-gen-1.2.1.scm which is 
; the script to build displacement map (v.1.2.1)
; written by Andrey Lebedenko (Lucidlook) 2009
;
; Flow implemented by Diego Nassetti to get the final result:
; 
; 1. Duplicate the image
; 2. Reflect vertically the duplicated image 
;
; 3. Generate the map using the original script, but indicating dimensions corresponding to the image 
;
; 4. on the duplicated image: Apply: Filters -> Map -> Displace
;     (indicating the two layers x e y of the map generated at point 3
;
; 5. File->Create->Misc->Horkan Tile Two Imaghes indicando originale, duplicata e posizionamento up
; ---------------------------------------------------------
;; ----------------------------------------------------------------
; 
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 3 of the License, or
; (at your option) any later version.  
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, you can view the GNU General Public
; License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html
; Alternatively you can write to the Free Software Foundation, Inc., 675 Mass
; Ave, Cambridge, MA 02139, USA.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; 
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   
(define (script-fu-full-dm-gen
				theOldImage
				theOldLayer
				mapNoise
				map3Dcorr
				mapNoBlue
				mapDecompose
				waterGradient
				doFlatten
                                smoothGradient
	)

	; Initialize an undo, so the process can be undone with a single undo
;    (gimp-image-undo-group-start theOldImage)

; Variable init
  (let* (

	(mapXuser (car (gimp-image-width theOldImage)))
	(mapYuser (car (gimp-image-height theOldImage)))
    (mask)
	(theImage)
	(newLayer)
    (theDuplicatedImage)
    (theDuplicatedLayer)
    (theRevertedImage)
    (theRevertedLayer)
    (theXLayer)
    (theYLayer)
	(theNewImage)
	(theNewLayer)
	(theWaterLayer)
	(theNoiseLayer)
	(theMergedLayer)
        (theMask)
	(red-component 0)
	(green-component 1)
	(blue-component 2)
	(alpha-component 5)
	(i 0)
	(RGBImage)
	(num_layers 0)
	(layer_ids)
	(current_layer)
	(theDisplay)
    )

(gimp-message-set-handler 2)
	(set! theImage (car (gimp-image-new mapXuser mapYuser RGB)))
	(set! newLayer (car (gimp-layer-new   theImage 
                                         mapXuser   mapYuser  RGB-IMAGE  "Displacement map"
                                         100   NORMAL-MODE )))


; Add new layer
	(gimp-image-add-layer theImage newLayer 0)
	(set! theDisplay (car (gimp-display-new theImage)))
   
	(gimp-context-set-background '(255 255 255))
	(gimp-context-set-foreground '(0 0 0))
	(gimp-drawable-fill newLayer BACKGROUND-FILL)
	(gimp-layer-set-lock-alpha newLayer TRUE)
	
	(while (< i mapNoise)
		(plug-in-rgb-noise RUN-NONINTERACTIVE theImage newLayer 0 1 0.2 0.2 0.2 0)
		(set! i (+ i 1))
	) ;end while
	
	(plug-in-gauss-iir 1 theImage newLayer 5 TRUE TRUE)

	(gimp-image-set-component-active theImage green-component FALSE)
	(gimp-image-set-component-active theImage blue-component FALSE)
	(plug-in-emboss 1 theImage newLayer 0 50.0 10 -5) ; horizontal
	
	(gimp-image-set-component-active theImage red-component FALSE)
	(gimp-image-set-component-active theImage green-component TRUE)
	(plug-in-emboss 1 theImage newLayer 90 50.0 10 -5) ; vertical

	(if (= mapNoBlue TRUE)
		(begin
		(gimp-image-set-component-active theImage red-component FALSE)
		(gimp-image-set-component-active theImage green-component FALSE)
		(gimp-image-set-component-active theImage blue-component TRUE)
		(gimp-edit-bucket-fill-full newLayer FG-BUCKET-FILL NORMAL-MODE 
                                                      100 255 FALSE TRUE SELECT-CRITERION-COMPOSITE 0 0)
		) ; end TRUE
	) ; end if
	(gimp-image-set-component-active theImage red-component TRUE)
	(gimp-image-set-component-visible theImage red-component TRUE)
	(gimp-image-set-component-active theImage green-component TRUE)
	(gimp-image-set-component-visible theImage green-component TRUE)

	(set! i 1)
	(while (<= i map3Dcorr)
		(gimp-perspective newLayer TRUE 0 0 
                                            mapXuser 0 (- 0 mapXuser) (* mapYuser 4) (* mapXuser 2) (* mapYuser 4))
		(gimp-layer-scale newLayer (* mapXuser 3) mapYuser FALSE)
		(gimp-layer-scale newLayer (* mapXuser 3) (+ mapYuser 10) FALSE)
		(gimp-image-crop (car(gimp-drawable-get-image newLayer)) mapXuser mapYuser 0 0)
		
		(set! i (+ i 1))
	) ; end while
	
	(gimp-drawable-transform-scale-default newLayer (- 0 mapXuser) 0 (* mapXuser 4) mapYuser TRUE TRUE)
	(gimp-image-crop (car(gimp-drawable-get-image newLayer)) mapXuser mapYuser 0 0)

	(set! mask (car (gimp-layer-create-mask newLayer ADD-WHITE-MASK)))
	(gimp-layer-add-mask newLayer mask)

	(gimp-edit-blend mask FG-BG-RGB-MODE NORMAL-MODE GRADIENT-LINEAR 
                                           100 0 REPEAT-NONE TRUE FALSE 1 0 TRUE 0 0 0 mapYuser)

	; mask to selection ( mask-to-selection ) :
	(gimp-selection-load mask)

	(gimp-image-set-component-active theImage red-component TRUE)
	(gimp-image-set-component-active theImage green-component FALSE)
	(gimp-image-set-component-active theImage blue-component FALSE)

	; fill with grey
	(gimp-context-set-foreground '(128 128 128))
	(gimp-edit-bucket-fill-full newLayer 
	                            FG-BUCKET-FILL NORMAL-MODE 100 255 FALSE TRUE SELECT-CRITERION-COMPOSITE 0 0)


	(gimp-selection-none theImage)
	(gimp-context-set-foreground '(0 0 0))
	(gimp-context-set-background '(255 255 255))
	
	(gimp-edit-blend mask FG-BG-RGB-MODE NORMAL-MODE GRADIENT-LINEAR 
                          100 0 REPEAT-NONE TRUE FALSE 1 0 TRUE 0 0 0 (round(* mapYuser 0.2)) )

	; mask to selection ( mask-to-selection ) :
	(gimp-selection-load mask)

	(gimp-image-set-component-active theImage red-component FALSE)
	(gimp-image-set-component-active theImage green-component TRUE)
	(gimp-image-set-component-active theImage blue-component FALSE)

	; fill with grey
	(gimp-context-set-foreground '(128 128 128))
	(gimp-edit-bucket-fill-full newLayer 
                                FG-BUCKET-FILL NORMAL-MODE 100 255 FALSE TRUE SELECT-CRITERION-COMPOSITE 0 0)
	
	(gimp-layer-remove-mask newLayer MASK-DISCARD)
	(gimp-selection-none theImage)
	
;(gimp-message (number->string newLayer))	

    (if (= mapDecompose TRUE) 
	(begin

		(set! RGBImage (car (plug-in-decompose RUN-NONINTERACTIVE
                                                  theImage newLayer "RGB" TRUE)))
		
	; since we decompose, we no longer need original map
;; (gimp-message (number->string theDisplay))
;; (gimp-message (number->string theImage))	
		(gimp-display-delete theDisplay)
	    ; delete blue channel/layer if requested by the user
		(set! num_layers (car (gimp-image-get-layers RGBImage)))
		(set! layer_ids (cadr (gimp-image-get-layers RGBImage)))
		(set! i 0)
	
		(while (< i num_layers)
			(set! current_layer (vector-ref layer_ids i))
			(if (= i 0) ; red
				(gimp-drawable-set-name current_layer "X")
  			)
			(if (= i 1) ; green
				(gimp-drawable-set-name current_layer "Y")

			)
			(if (= i 2) ; blue
				(if (= mapNoBlue TRUE)
					(gimp-image-remove-layer RGBImage current_layer)
				)

			)

			(if (= i 0) ; red
                        (set! theXLayer current_layer))
			(if (= i 1) ; green
                        (set! theYLayer current_layer))

			(set! i (+ i 1))
		) ; end while
;
	;;;;;;;;;;	(gimp-display-new RGBImage)
	) ; end begin
	) ; end if (TRUE)
        (if (= mapDecompose FALSE) 
	(begin
        (set! theXLayer newLayer) 
        (set! theYLayer newLayer)
;(gimp-message (number->string newLayer))	
;(gimp-message (number->string theXLayer))
;(gimp-message (number->string theYLayer))
    ;Finish the undo group for the process and enable all channels
		(gimp-image-set-component-active theImage red-component TRUE)
		(gimp-image-set-component-active theImage green-component TRUE)
		(gimp-image-set-component-active theImage blue-component TRUE)
	 ) ; end begin
	) ; end if (FALSE)



    (set! theNewImage (car (gimp-image-new mapXuser (* 2 mapYuser) RGB)))
    (set! theNewLayer (car (gimp-layer-new-from-drawable theOldLayer theNewImage)))
    (gimp-image-add-layer theNewImage theNewLayer -1)

    (set! theRevertedLayer (car (gimp-layer-new-from-drawable theOldLayer theNewImage)))
    (gimp-image-add-layer theNewImage theRevertedLayer -1)
    (gimp-drawable-transform-flip-simple theRevertedLayer 1 TRUE 0 FALSE)
    (gimp-layer-translate theRevertedLayer 0 mapYuser)

    (set! theWaterLayer (car (gimp-layer-new theNewImage mapXuser mapYuser RGB 
                                         "Water Layer" 90 0)))
 
    (gimp-image-add-layer theNewImage theWaterLayer -1)
	
    (gimp-context-set-gradient waterGradient)
    (gimp-edit-blend theWaterLayer 3 0 0 90 0 0 FALSE FALSE 0 0 TRUE 
                     (/ mapXuser 2) 0 (/ mapXuser 2) mapYuser )
    (gimp-layer-translate theWaterLayer 0 mapYuser)
	
	(set! theNoiseLayer (car (gimp-layer-new-from-drawable theWaterLayer theNewImage)))
	(gimp-image-add-layer theNewImage theNoiseLayer -1)
	(plug-in-solid-noise 1 theNewImage theNoiseLayer FALSE TRUE 0 15 6 16)
	(gimp-brightness-contrast theNoiseLayer 0 33)
	(gimp-layer-set-mode theNoiseLayer 3)
	(gimp-layer-set-opacity theNoiseLayer 90)
	
	(set! theMergedLayer (car (gimp-image-merge-down theNewImage theNoiseLayer 0)))
	(gimp-layer-set-opacity theMergedLayer 85)
        (set! theMask (car (gimp-layer-create-mask theMergedLayer 0)))
        (gimp-layer-add-mask theMergedLayer theMask)
        (gimp-context-set-gradient smoothGradient)
        (gimp-edit-blend theMask 3 0 0 90 0 0 FALSE FALSE 0 0 TRUE 
                     (/ mapXuser 2) 0 (/ mapXuser 2) mapYuser )
        
	
	   
	(set! theDisplay (car (gimp-display-new theNewImage)))

    (plug-in-displace 1 theNewImage theRevertedLayer 20.0 20.0 1 1 theXLayer theYLayer 1)
	
	(if (= doFlatten TRUE)
		(gimp-image-flatten theNewImage)
		)

    ;Ensure the updated image is displayed now
    (gimp-displays-flush)
)
)

(script-fu-register "script-fu-full-dm-gen"
            _"<Image>/Script-Fu/Map/Full Water reflection with map"
            "Generate a new image implementing the water reflection with mapping"
            "Diego Nassetti"
            "Diego Nassetti"
            "03.2009-05.2014"
            "*"
            SF-IMAGE        "Image"     0
            SF-DRAWABLE     "Drawable"  0
    	    SF-ADJUSTMENT   _"Noise level"     '(6 1 20 1 1 0 0)
			SF-ADJUSTMENT   _"3D corrections"     '(2 0 10 1 1 0 0)
            SF-TOGGLE       _"Delete blue channel"	FALSE
            SF-TOGGLE       _"Decompose channels into layers"	TRUE
            SF-GRADIENT     _"Water Gradient" "Caribbean Blues"
            SF-TOGGLE       _"Flatten Image"	FALSE
            SF-GRADIENT     _"Smoothing Gradient" "FG to BG (RGB)"
)

