; Script to implement Issabella Tutorial on Gimpchat (see Manhattan topic)
;
; Flow implemented to get the final result:
; 
; 1. Duplicate the image
; 2. Add a new layer with the same image
; 3. Compress the added layer to a square (if requested by the user)
; 4. Map this layer to a sphere (the user can modify space and rotations)
; 5. Zoom blur the background image
; 6. Flatten the image (is requested by the user)
; ----------------------------------------------------------------
; ----------------------------------------------------------------
; 
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 3 of the License, or
; (at your option) any later version.  
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, you can view the GNU General Public
; License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html
; Alternatively you can write to the Free Software Foundation, Inc., 675 Mass
; Ave, Cambridge, MA 02139, USA.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; 
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   
(define (script-fu-globe-view
				theOldImage
				theOldLayer
				globeSpace
				rotateX
				rotateY
				rotateZ
				doScale
				doFlatten
	)


; Variable init
  (let* (

	(imageWidth (car (gimp-image-width theOldImage)))
	(imageHeight (car (gimp-image-height theOldImage)))
    (theDuplicatedImage)
    (theDuplicatedLayer)
	(theNewImage)
	(theNewLayer)
	(theGlobeLayer)
	(globeWidth)
	(globeHeight)
	(xOffset)
	(theMergedLayer)
	(theDisplay)
    )

(gimp-message-set-handler 2)
    (set! theNewImage (car (gimp-image-duplicate theOldImage)))
	
	; Initialize an undo, so the process can be undone with a single undo
    (gimp-image-undo-group-start theNewImage)

    (set! theNewLayer (car (gimp-image-get-active-layer theNewImage))) 
    (set! theGlobeLayer (car (gimp-layer-new-from-drawable theNewLayer theNewImage)))
	(gimp-image-insert-layer theNewImage theGlobeLayer 0 0)
	(gimp-layer-add-alpha theGlobeLayer)
    (gimp-image-set-active-layer theNewImage theGlobeLayer)
	(if (< imageHeight imageWidth)
		(begin
		(set! globeWidth imageHeight)
		(set! globeHeight imageHeight)
		) ;else
		(begin
		(set! globeWidth imageWidth)
		(set! globeHeight imageWidth)
		)	
	)		
	(if (= doScale TRUE)
		(gimp-layer-scale theGlobeLayer globeWidth globeHeight 1)
	)
	
	(plug-in-map-object RUN-NONINTERACTIVE theNewImage theGlobeLayer 
         1                     ; Type of mapping (0=plane,1=sphere,2=box,3=cylinder)
         0.5 0.5 2.0           ; viewpoint
         0.5 0.5 globeSpace    ; object pos (last acts as a percentage of the space)
         1.0 0.0 0.0           ; first axis
         0.0 1.0 0.0           ; 2nd axis
         rotateX rotateY rotateZ       ; axis rotation (were all 0.0)
         1 '(255 255 255)      ; Type of lightsource (0=point,1=directional,2=none)and Lightsource color (r,g,b) 
         -0.5 -0.5 2           ; light position
         -1.0 -1.0 1.0         ; light direction
         0.3 1.2 0.7 0.0 27.0  ; material (amb, diff, refl, spec, high)
         TRUE                  ; antialias
         FALSE                 ; tile
         FALSE                 ; new image
         TRUE                  ; transparency
         .25                   ; Sphere/cylinder radius (only used when maptype=1 or 3)
         0.5 0.5 0.5           ; box size
         1.00                  ; unused parameter Cylinder length
         -1                    ; Box front face (set these to -1 if not used)
         -1                    ; Box back face
         -1                    ; Box top face 
         -1                    ; Box bottom face
         -1                    ; Box left face
         -1                    ; Box right face
         -1                    ; Cylinder top face (unused parameter)
         -1)                   ; Cylinder bottom face (unused parameter)
		 
	(gimp-image-set-active-layer theNewImage theNewLayer)	
	(plug-in-mblur 1 theNewImage theNewLayer 2 20 0 (/ imageWidth 2) (/ imageHeight 2))
	
    (set! theDisplay (car (gimp-display-new theNewImage)))
   
;(gimp-message (number->string theNewImage))
;(gimp-message (number->string theNewLayer))	
;(gimp-message (number->string theGlobeLayer))
;(gimp-message (number->string globeWidth))
;(gimp-message (number->string globeHeight))

	
	(if (= doFlatten TRUE)
	    (gimp-image-flatten theNewImage)
		)

    ;Ensure the updated image is displayed now
    (gimp-displays-flush)

    (gimp-image-undo-group-end theNewImage)
)
)

(script-fu-register "script-fu-globe-view"
            _"<Image>/Script-Fu/Map/Globe View of an Image"
            "Create a Globe View of a Panorama Image ..........."
            "Diego Nassetti"
            "Diego Nassetti"
            "17/05/2014"
            "*"
            SF-IMAGE        "Image"     0
            SF-DRAWABLE     "Drawable"  0
    	    SF-ADJUSTMENT   _"Space reserved" '(0.65 0.00 1.00 0.05 0.05 2 0) ;default, min, max, incr, incr, dec, type
			SF-ADJUSTMENT   _"Rotate X" '(0 -90 90 1 5 0 0) ;default, min, max
			SF-ADJUSTMENT   _"Rotate Y" '(0 -90 90 1 5 0 0) ;default, min, max
			SF-ADJUSTMENT   _"Rotate Z" '(0 -90 90 1 5 0 0) ;default, min, max
            SF-TOGGLE       _"Want to scale?"	TRUE
			SF-TOGGLE       _"Flatten Image?"	FALSE
)

