; following Oregonian's tutorial on Double Bump Map
; found at link: http://gimpchat.com/viewtopic.php?f=23&t=12
; this is intended for GIMP 2.6
; author: Tin Tran
; date: 2014
(define (script-fu-double-bump-map image layer 
              color
              grow-distance
			  blur-radius
			  bump-azimuth
			  bump-elevation
			  bump-depth
			  ;shadow settings
			  shadow-offset-X   ;offset x
			  shadow-offset-Y   ;offset y
			  shadow-blur-radius ;blur-radius
			  shadow-color       ;color
	          shadow-opacity     ;opacity
         )
	(let* 
	  (
	  (dummy-variable-to-make-let-happy 0)
	  (image-width)
	  (image-height)
	  
	  ;vectors/paths related variables to create our own path and select it to cut after blur
	  (get-vectors-returned-values)
	  (vectors-count)
	  (vectors-array)
	  (current-vector)
	  
	  (color-layer)
	  
	  ) ;end of variable declaration
	  (gimp-image-undo-group-start image)                   ;undo-group in one step
	  
	  
	  (set! image-width (car (gimp-image-width image)))
	  (set! image-height (car (gimp-image-height image)))
      ;START
	  (gimp-layer-resize-to-image-size layer)
	  ;(gimp-image-select-item image CHANNEL-OP-REPLACE layer) ;2.8
	  (gimp-selection-layer-alpha layer) ;2.6
	  (gimp-edit-fill layer WHITE-FILL)
	  
	  ;grow selection, invert, the create vector path
	  (gimp-selection-grow image grow-distance)
	  (gimp-selection-invert image)
	  (plug-in-sel2path RUN-NONINTERACTIVE image layer)
	  (set! get-vectors-returned-values (gimp-image-get-vectors image))
	  (set! vectors-count (car  get-vectors-returned-values))
	  (set! vectors-array (cadr get-vectors-returned-values))
	  (set! current-vector (aref vectors-array 0))      ;top path, our own created from selection
	  
	  ;blurs white bump map
	  (gimp-selection-none image)
	  (plug-in-gauss RUN-NONINTERACTIVE image layer
	                                        blur-radius blur-radius
											1 ;1 RLE or 0 IIR
	                                        )
											
	  ;creates new colored layer to bump map
	  (set! color-layer (car (gimp-layer-new image image-width image-height
	                           RGBA-IMAGE "Color" 100 NORMAL-MODE)))  ;creates layer
	  ;insert above current layer  
      ;(gimp-image-insert-layer image color-layer 0 (car (gimp-image-get-item-position image layer)))	;2.8
      (gimp-image-add-layer image color-layer (car (gimp-image-get-layer-position image layer)))	  
	  (gimp-image-set-active-layer image color-layer)
	  (gimp-selection-all image)
	  (gimp-context-set-foreground color)
	  (gimp-edit-fill color-layer FOREGROUND-FILL)
	  
	  ;perform bump map 1st time
	  (plug-in-bump-map 1 image color-layer layer 
	                          bump-azimuth ;Azimuth
							  bump-elevation  ;Elevation
							  bump-depth  ;Depth
							  0   ;X-offset
							  0   ;Y-offset
							  0   ;water level
							  0   ;ambient
							  TRUE ;compensate for darkening
	                          FALSE ;invert
							  0    ;type of map LINEAR (0), SPHERICAL (1), SINUSOIDAL (2)
	   )
	   
	   ;cut out color layer using previously created path
	   ;(gimp-image-select-item image CHANNEL-OP-REPLACE current-vector) 2.8
	   (gimp-vectors-to-selection current-vector CHANNEL-OP-REPLACE 
	                                                 TRUE ;antialias
													 FALSE ;feather selection
													 0    ;feather radius x
													 0    ;feather radius y
													 )
	   (gimp-edit-cut color-layer)
	   (gimp-selection-none image)
	   ;bump map 2nd time
	   (plug-in-bump-map 1 image color-layer layer 
	                          bump-azimuth ;Azimuth
							  bump-elevation  ;Elevation
							  bump-depth  ;Depth
							  0   ;X-offset
							  0   ;Y-offset
							  0   ;water level
							  0   ;ambient
							  TRUE ;compensate for darkening
	                          FALSE ;invert
							  0    ;type of map LINEAR (0), SPHERICAL (1), SINUSOIDAL (2)
	   )
	   
	   ;drops shadow of color-layer
	   (script-fu-drop-shadow image color-layer
	                               shadow-offset-X   ;offset x
								   shadow-offset-Y   ;offset y
								   shadow-blur-radius ;blur-radius
								   shadow-color       ;color
	                               shadow-opacity     ;opacity
								   TRUE ;allow resizing
	   )
	  ;DONE
	  
	
	(gimp-image-undo-group-end image)                     ;undo group in one step
	(gimp-displays-flush)
    ) ;end of let*
) ;end of define

(script-fu-register
  "script-fu-double-bump-map"         ;function name
  "<Image>/Script-Fu/Double Bump Map"    ;menu register
  "Creates Double Bump Map effect on your art/text/shapes."       ;description
  "Tin Tran"                          ;author name
  "copyright info and description"         ;copyright info or description
  "2014"                          ;date
  "RGB*, GRAY*"                        ;mode
  SF-IMAGE      "Image" 0                   
  SF-DRAWABLE   "Layer" 0
  SF-COLOR      "Color"          '(0 129 229)
  SF-ADJUSTMENT "Grow Distance(from border to cut)" '(2 0 100 1 10 0 0)
  SF-ADJUSTMENT "Bump Map Blur Radius"             '(3 0 100 1 10 0 0)
  SF-ADJUSTMENT "Bump Azimuth"             '(135 0 360 1 10 0 0)
  SF-ADJUSTMENT "Bump Elevation"           '(33 0 100 1 10 0 0)
  SF-ADJUSTMENT "Bump Depth"               '(35 0 100 1 10 0 0)
  
  SF-ADJUSTMENT "Shadow Offset X" '(4 0 100 1 10 0 0)
  SF-ADJUSTMENT "Shadow Offset Y" '(4 0 100 1 10 0 0)
  SF-ADJUSTMENT "Shadow Blur Radius" '(15 0 100 1 10 0 0)
  SF-COLOR      "Shadow Color"     '(0 0 0)
  SF-ADJUSTMENT "Shadow Opacity"   '(60 0 100 1 10 0 0)
  
)