; EnviroMap V1.5.1 
;
; Created by GnuTux 
; Comments directed to http://gimpchat.com or http://gimpscripts.com
;
; License: GPLv3
;    This program is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;    GNU General Public License for more details.
;
;    To view a copy of the GNU General Public License
;    visit: http://www.gnu.org/licenses/gpl.html
;
; ------------
;| Change Log |
; ------------ 
; V1.0 - Initial Release 
; V1.1 - Add Cloud Map - Option To Blend Map with Gradient using Paint Modes - Make Blur & Autostretch HSV Configurable
; V1.2 - Add Gradient Blend Shape & Direction - Tweak UI Layout 
; V1.3 - Add Option To Use Active Gradient rather than Selected Gradient - Option To Gauss Blur Final Emap
; V1.4 - True Random Number Generation
; V1.5 - Add Option To Map Environment Map To Sphere (with Iteractive Object Map Option)
; V1.5a - Bug that flattened edges on sphere   
;
;
;Define Main Procedure
;

(define (script-fu-environment-map 

               img    ; Image
          drawable    ; Drawable
          map-type    ; Environment Map Type
        paint-mode    ; Selected Paint Mode For Use WIth Blend Plug-in
   blend-direction    ; Blend Direction
       blend-shape    ; Blend Shape
     emap-gradient    ; Selected Gradient For Environment Map
      detail-level    ; Detail Level For Solid Noise Plug-in 
      texturesizex    ; Drawable Width
      texturesizey    ; Drawable Height 
       blur-radius    ; Gaussian Blur Radius 
          auto-hsv    ; Autostretch HSV Flag
         turbulent    ; Turbulance Flag
         spherical    ; Genrate Spherical Map Flag
         interactive) ; Interactive Spherical Map Flag

;
;Declare Variables
;
    (let* 
    (
	  (image-height 0)       ; Drawable Height For Blend Plug-in
      (image-width 0)        ; Drawable Width For Blend Plug-in
      (xblend-start 0)       ; X Blend Starting Point
      (yblend-start 0)       ; X Blend Starting Point
      (xblend-end 0)         ; X Blend Ending Point
      (yblend-end 0)         ; X Blend Ending Point
    )
;
; Save Context
; 
(gimp-context-push)

; Start Undo Group

(gimp-undo-push-group-start img)

;
; Call Solid Noise Plug-in
;
(set! *seed* (car (gettimeofday))) ; Random Number Seed From Clock (*seed* is global)
(random-next)                      ; Next Random Number Using Seed

      (plug-in-solid-noise 
                       1         ; Non-Interactive
                     img         ; Input Image
                drawable         ; Imput Drawable
                    TRUE         ; Tileable
               turbulent         ; Turbulent Noise
           (random-next)         ; Generate Random Number
            detail-level         ; Detail Level
            texturesizex         ; Horizontal Texture Size
            texturesizey         ; Vertical Texture Size
      )

;
; Auto-stretch HSV
;
(if (= auto-hsv TRUE)
    (begin  
      (plug-in-autostretch-hsv 1 img drawable)
    )
) ;endif

;
; Gaussian Blur Before Creating eMap
;
(plug-in-gauss                 
             1     ; Non-interactive 
           img     ; Image to apply blur 
      drawable     ; Layer to apply blur
             5     ; Blur Radius x  
             5     ; Blue Radius y 
             0)    ; Method (IIR=0 RLE=1)

;
; Set Specified Gradient Active Unless Active Gradient Is Selected
;
(if (< map-type 2)
    (begin  
       (gimp-context-set-gradient emap-gradient)
     )
) ;endif

;
; Environment Map Type: Topographical
;
(if (or (= map-type 0) (= map-type 2))
    (begin  
      (plug-in-gradmap 1 img drawable)                 ; Map Gradient
      (gimp-layer-set-name drawable "EnviroMap: Topo") ; Name it
     )
) ;endif

;
; Environment Map Type: Cloud
;
(if (or (> paint-mode 0) (= map-type 1) (= map-type 3))
    (begin  

     (set! image-height (car (gimp-drawable-height drawable)))     ; Get Height 
     (set! image-width (car (gimp-drawable-width drawable)))       ; Get Width

;
; Calculate Gradient Blend Start & End Points
;

      (cond ((= blend-direction 0)         ; Top-Bottom 
          (set! xblend-start 0)            ; X Blend Starting Point
          (set! yblend-start 0)            ; Y Blend Starting Point
          (set! xblend-end 0)              ; X Blend Ending Point
          (set! yblend-end image-height)   ; Y Blend Ending Point
          )
          ((= blend-direction 1)           ; Bottom-Top
          (set! xblend-start 0)            ; X Blend Starting Point
          (set! yblend-start image-height) ; Y Blend Starting Point
          (set! xblend-end 0)              ; X Blend Ending Point
          (set! yblend-end 0)              ; Y Blend Ending Point
          )
          ((= blend-direction 2)           ; Left-Right
          (set! xblend-start 0)            ; X Blend Starting Point
          (set! yblend-start 0)            ; Y Blend Starting Point
          (set! xblend-end image-width)    ; X Blend Ending Point
          (set! yblend-end 0)              ; Y Blend Ending Point
          )
          ((= blend-direction 3)           ; Right-Left
          (set! xblend-start image-width)  ; X Blend Starting Point
          (set! yblend-start 0)            ; Y Blend Starting Point
          (set! xblend-end 0)              ; X Blend Ending Point
          (set! yblend-end 0)              ; Y Blend Ending Point
          )
          ((= blend-direction 4)           ; Diag-Top-Left
          (set! xblend-start 0)            ; X Blend Starting Point
          (set! yblend-start 0)            ; Y Blend Starting Point
          (set! xblend-end image-width)    ; X Blend Ending Point
          (set! yblend-end image-height)   ; Y Blend Ending Point
          )
          ((= blend-direction 5)           ; Diag-Top-Right
          (set! xblend-start image-width)  ; X Blend Starting Point
          (set! yblend-start 0)            ; Y Blend Starting Point
          (set! xblend-end 0)              ; X Blend Ending Point
          (set! yblend-end image-height)   ; Y Blend Ending Point
          )
          ((= blend-direction 6)           ; Diag-Bottom-Left
          (set! xblend-start 0)            ; X Blend Starting Point
          (set! yblend-start image-height) ; Y Blend Starting Point
          (set! xblend-end image-width)    ; X Blend Ending Point
          (set! yblend-end 0)              ; Y Blend Ending Point
          )
          ((= blend-direction 7)           ; Diag-Bottom-Right
          (set! xblend-start image-width)  ; X Blend Starting Point
          (set! yblend-start image-height) ; Y Blend Starting Point
          (set! xblend-end 0)              ; X Blend Ending Point
          (set! yblend-end 0)              ; Y Blend Ending Point
          )
;          (else
;                 ; For later use.. 
;          )
       ) ;end cond
;
; Check Paint Mode
;

     (if (= paint-mode 0)
            (begin  
              (set! paint-mode 18)   ; Normal Makes No Sense For Cloud Map So Set To Hard Light As Default
            )
      ) ;endif

             (gimp-edit-blend        ; Blend (Gradient)
                       drawable      ; Drawable - (The affected drawable)
                        CUSTOM       ; Int32 - Blend Type (FG-BG-RGB-MODE (0))
                        paint-mode   ; Int32 - Paint Mode (NORMAL-MODE (0)) 
                        blend-shape  ; Int32 - Gradient Type (LINEAR (0))
                        100          ; Float - Opacity (0 - 100)
                        0            ; Float - Offset ( >= 0) 
                        0            ; Int32 - Repeat (NONE (0))
                        FALSE        ; Int32 - Reverse (TRUE or FALSE)
                        TRUE         ; Int32 - Supersample (TRUE or FALSE)
                        3            ; Int32 - Supersampling Recursion  Dept (1 - 9)
                        0.20         ; Float - Supersampling threshold (0 <= 4)
                        TRUE         ; Int32 - Dither (TRUE or FALSE)
                        xblend-start ; Int32 - X Blend Starting Point
                        yblend-start ; Int32 - Y Blend Starting Point
                        xblend-end   ; Int32 - X Blend Ending Point  
                        yblend-end   ; Int32 - Y Blend Ending Point
             ) ; End Blend

             (gimp-layer-set-name drawable "EnviroMap: Cloud")          ; Name Environment Map
     )
) ;endif


;
; Ensure Environment Map Does Not Have An Alpha Channel So Flatten (If Not Spherical or map to Object)
;
(if (= spherical FALSE)
    (begin  
      (gimp-layer-flatten drawable)
    )
)
;
; Gaussian Blur The Final Map
;
(if (> blur-radius 0)
    (begin  
      (plug-in-gauss                 
                   1     ; Non-interactive 
                 img     ; Image to apply blur 
            drawable     ; Layer to apply blur
         blur-radius     ; Blur Radius x  
         blur-radius     ; Blue Radius y 
                   0     ; Method (IIR=0 RLE=1)
      )
    )
) ;endif

;
; Genrate Sphercial Map or Map Object Interacively
;

(if (or (= spherical TRUE) (= interactive 1))
    (begin
          (if (= interactive 0)       ; Flip Interactive Flag
                ; True 
                (set! interactive 1)
                ; Else
                (set! interactive 0)   
          )

     (gimp-layer-add-alpha drawable) ; Add alpha channel drawable
      
;
;  Map Object
;
     (plug-in-map-object interactive img drawable
         1                       ; Type of mapping (0=plane,1=sphere,2=box,3=cylinder)
         0.5 0.5 2.0             ; Viewpoint
         0.5 0.5 0.0             ; Object pos
         1.0 0.0 0.0             ; First axis
         0.0 1.0 0.0             ; 2nd axis
         0.0 0.0 0.0             ; Axis rotation
         0 '(255 255 255)        ; Type of lightsource (0=point,1=directional,2=none)and Lightsource color (r,g,b)
         -0.5 -0.5 2.0           ; Light position
         -1.0 -1.0 1.0           ; Light direction
         0.3 1.0 0.45 0.35 30.0  ; Material (amb, diff, refl, spec, high)
         TRUE                    ; Antialias
         FALSE                   ; Tile
         FALSE                   ; New image
         TRUE                    ; Transparency
         .44                     ; Sphere/cylinder radius (only used when maptype=1 or 3)
         0.5 0.5 0.5             ; Box size
         1.00                    ; Unused parameter Cylinder length
         -1                      ; Box front face (set these to -1 if not used)
         -1                      ; Box back face
         -1                      ; Box top face
         -1                      ; Box bottom face
         -1                      ; Box left face
         -1                      ; Box right face
         -1                      ; Cylinder top face (unused parameter)
         -1)                     ; Cylinder bottom face (unused parameter)
    )
) ;endif


;
; End Undo Group
;
(gimp-undo-push-group-end img)

;
; Update display
;
(gimp-displays-flush)

;
; Restore Context 
;
(gimp-context-pop)

) ; End let
) ; End Main Procedure 

;
; Register Environment Map Script
;
(script-fu-register "script-fu-environment-map"
			"<Image>/Filters/Render/Environment Map..."
			"Render An Environment Map"
			"GnuTux - http://gimpchat.com"
			"GnuTux - GPLv3"
			"August, 2013"
			"RGB*"
			SF-IMAGE       "Image"                    0
			SF-DRAWABLE    "Drawable"                 0
            SF-OPTION     _"Map Type"                 '("Topographical" "Cloud" "Topographical (Active Gradient)" "Cloud (Active Gradient)")
            SF-OPTION      "Blending Mode"            '("Normal" "Dissolve" "Behind" "Multiply" "Screen" "Overlay" "Difference" "Addition" "Subtract" "Darken" "Lighten" "Hue" "Saturation" "Color" "Value" "Divide" "Dodge" "Burn" "Hard Light" "Soft Light" "Grain Extract" "Grain Merge")
            SF-OPTION     _"Blend Direction"          '("Top-Botton" "Bottom-Top" "Left-Right" "Right-Left" "Diag-Top-Left" "Diag-Top-Right" "Diag-Bottom-Left" "Diag-Bottom-Right")
            SF-OPTION     _"Blend Shape"              '("Linear""Bilinear" "Radial" "Square" "Conical-Symmetric" "Conical-Asymmetric" "Shapeburst-Angular" "Shapburst-Spherical" "Shapeburst-Dimpled" "Spiral-CW" "Spiral-CCW")
            SF-GRADIENT   _"Select Gradient"          "Golden"
            SF-ADJUSTMENT _"Detail Level"             '(1 0 5 1 1 0 0)
            SF-ADJUSTMENT _"Horizontal Texture Size"  '(2.8 1 5 .1 .5 1 0)
            SF-ADJUSTMENT _"Vertical Texture Size"    '(2.8 1 5 .1 .5 1 0)
            SF-ADJUSTMENT _"Blur On Final Map"        '(0 0 30 1 5 0 0)
            SF-TOGGLE     _"Autostretch HSV"          TRUE
            SF-TOGGLE     _"Turbulent Noise"          FALSE
            SF-TOGGLE     _"Generate Spherical Map"   FALSE
            SF-TOGGLE     _"Interactive Map Object"   FALSE
            
) ;End register 
