'''

 Filter to create a NEW image applying ROD's style for detail effects to the INPUT DRAWABLE
 ..........................................................................................
 
 Release 1.0   initial
 
 Flow implemented to get the final result:      
 
 1. save current context
 2. prepare the new image, copying the source drawable into the new BG
    add noise, if chosen by the user (useful on people images)
 3. duplicate the BG, name it LumaInvertedLayer, SOFTLIGHT with opacity=100%
 4. apply Gimp Luma Invert (script-fu-invert-YUV) on the above layer
 5. new from visible, name it Visible1Layer, NORMAL with opacity=100%
 6. duplicate the Visible1Layer, name it NeonLayerLayer, GRAIN_MERGE with opacity=100%
    apply Gimp Neon then apply Gimp Unsharp Mask (user parameter for details level)
 7. duplicate the BG on top, and call it BGonTop, SOFTLIGHT with opacity 100%
 8. flatten the new image if requested
 
 
 ----------------------------------------------------------------
 COPYRIGHT NOTICE
 ----------------------------------------------------------------
 
 This program is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation either version 3 of the License, or
 (at your option) any later version.  
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program if not, you can view the GNU General Public
 License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html
 Alternatively you can write to the Free Software Foundation, Inc., 675 Mass
 Ave, Cambridge, MA 02139, USA.
 
 -------------------------------------------------------------------
 other info
 -------------------------------------------------------------------
  
'''
#!/usr/bin/env python
from gimpfu import *
import decimal
import random


def RodStyleDetails (inImage, inDrawable, widthheight,
                     details, addNoise,
                     doFlatten,
                     ) :

    pdb.gimp_context_push
    currFG = pdb.gimp_context_get_foreground()
    currBG = pdb.gimp_context_get_background()
    
    width = inImage.width
    height = inImage.height
    type = inImage.base_type
    
    name = inDrawable.name
    if '.' in name:
        split = name.split(".", 1)
        name = split[0]
    suffix = ""
    
    # calculate scale factor for the new output image
    if widthheight == 0 :   # means the user wants to have same size as input
        scale = 1
    else :     
        optiWidthHeight = widthheight       # size to work on to get best results
        realWidthHeight = width+height      # real size of the input image
        scale = decimal.Decimal(optiWidthHeight)/decimal.Decimal(realWidthHeight)      # ratio for scaling

    # Set up an undo group, so the operation will be undone in one step.
    pdb.gimp_image_undo_group_start(inImage)
    # fix all required context settings 
    pdb.gimp_context_set_antialias(True)
    pdb.gimp_context_set_feather(False)
    pdb.gimp_context_set_feather_radius(0, 0)
    pdb.gimp_context_set_sample_merged(False) 
    pdb.gimp_context_set_sample_criterion(SELECT_CRITERION_COMPOSITE) 
    pdb.gimp_context_set_sample_threshold(0.00)
    pdb.gimp_context_set_sample_transparent(False)       # beware, if TRUE may change the area selected!
    pdb.gimp_context_set_interpolation(INTERPOLATION_LANCZOS)
    
    newwidth=int(width*scale)
    newheight=int(height*scale)
    
    #  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< 
    newImage = pdb.gimp_image_new (newwidth, newheight, type)                    
    newBG = pdb.gimp_layer_new_from_drawable(inDrawable, newImage)     # background layer
    newBG.mode = NORMAL_MODE
    newBG.name = "Background Image Layer"
    newBG.opacity = 100.0 
    newImage.add_layer (newBG, 0)
    pdb.gimp_layer_scale(newBG, newwidth, newheight, False)
    alpha=pdb.gimp_drawable_has_alpha(newBG)
    if (alpha==False):
        pdb.gimp_layer_add_alpha (newBG)    #  RGBA
    
    if addNoise == True :
        # adding noise 
        newBG = pdb.plug_in_hsv_noise (newImage, newBG, 8, 30, 30, 30)
        newBG = pdb.gimp_image_get_active_layer (newImage)
        suffix = suffix+", noise added"
        
    
    #  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< 
    LumaInvertedLayer = newBG.copy()
    LumaInvertedLayer.mode = SOFTLIGHT_MODE
    LumaInvertedLayer.name = "Luma Inverted Layer"
    LumaInvertedLayer.opacity = 100.0 
    newImage.add_layer (LumaInvertedLayer, 0)
    LumaInvertedLayer = newImage.active_layer
    pdb.script_fu_invert_YUV (newImage, LumaInvertedLayer) 
    LumaInvertedLayer = newImage.active_layer
        
    #  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< 
    Visible = pdb.gimp_layer_new_from_visible (newImage, newImage, "Visible Layer  ")
    Visible.mode = NORMAL_MODE
    Visible.opacity = 100.0 
    newImage.add_layer (Visible, 0)
        
    #  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< 
    NeonLayer = Visible.copy()
    NeonLayer.mode = GRAIN_MERGE_MODE
    NeonLayer.name = "Neon Layer"
    NeonLayer.opacity = 100.0 
    newImage.add_layer (NeonLayer, 0)
    pdb.plug_in_neon (newImage, NeonLayer, 1.00, 1.00)
    #pdb.plug_in_unsharp_mask (newImage, NeonLayer, 0.10, 3.75, 0)
    pdb.plug_in_unsharp_mask (newImage, NeonLayer, 0.10, details, 0)       # try
        
    #  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< 
    BGonTop = pdb.gimp_layer_new_from_drawable (newBG, newImage) 
    BGonTop.mode = SOFTLIGHT_MODE
    BGonTop.name = "Dup BG Layer"
    BGonTop.opacity = 100.0 
    newImage.add_layer (BGonTop, 0)
    #  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< 
    
    
    if (doFlatten == True) :
        pdb.gimp_image_flatten(newImage)
        flattenedLayer = pdb.gimp_image_get_active_layer (newImage)
        pdb.gimp_item_set_name(flattenedLayer, "Rod Style Details on "+name+suffix)
    
    pdb.gimp_display_new(newImage)
    
    pdb.gimp_displays_flush
   
    # Close the undo group.
    pdb.gimp_image_undo_group_end(inImage)
    
    pdb.gimp_context_pop
    pdb.gimp_context_set_foreground(currFG)
    pdb.gimp_context_set_background(currBG)
    
    if doFlatten == True :
        return(flattenedLayer)
    else :
        return()


# This is the plugin registration function
register(
    "RodStyleDetails",    
    "to create a new image applying Rod's Style details effects to the source drawable",   
    "This script creates a new image applying Rod's Style details effects to the source drawable",
    "Diego", 
    "Diego Nassetti ", 
    "September 2016",
    "RodStyleDetails (use noise for people!)", 
    "RGB*", 
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "idrawable", "Input drawable", None),
      (PF_ADJUSTMENT, "size", "Width+Height of New Image (0=same as input)", 2100, (0, 9600, 300)),
      (PF_ADJUSTMENT, "details", "Details amount", 3.75, (0.10, 10.00, 0.05)),
      (PF_TOGGLE, "addnoise", "Adding noise? (preprocess)", True),
      (PF_TOGGLE, "flatten", "Flatten Image?", False),
    ], 
    [
      (PF_DRAWABLE, "odrawable", "Output drawable", None),
    ],
    RodStyleDetails,
    menu="<Image>/Diego/Enhance",
    )

main()
