'''
 Filter to implement the creation of a MOSAIC IMAGE LYLE-STYLE
 ............................................................. 
 
 Release 1.0   initial
 
 
Flow implemented to get the final result:     
 
 1. Create the work image on the base of the size entered and the aspect ratio of the original image
 2. Stretch the colours using an ancillary function (embedded)
 3. Add a Layer (duplicate BG) and run G'MIC Isophotes with coloured BG
 4. Add a Layer (duplicate BG) and run G'MIC Isophotes with transparent BG
    and filter to have only black or transparent pixels using an ancillary function (embedded)
 6. Add a Layer (duplicate of Transparent Isophotes), select the Transparent areas
    and run Gimp RobA Mosaic tile helper
 7. Create a work brush of 1 pixel with max specing (1000 which corresponds to 10 pixels)
 8. Add a layer (duplicate of RobA output), stroke both the Contour generated by Isophote
    and the Path generated by RobA
 9. Fill the empty space around the generated dots with White, remove the path and the selection,  
 10.Add a Layer (duplicate of the Dots layer) and run G'MIC Skeleton to create the tiles
 11.Duplicate the Isophotes contours layer above
 12.Create a new layer combining the whole grid
 13.Duplicate the Isophotes colours layer above (grain-merge)
 14.Duplicate it again (hard-light)
 15.Duplicate the BG layer above (screen)
 16.Duplicate it again
 17.Analyze the BG Layer luminosity and set the top layer mode to:
    - screen if the image is very dark
    - burn if the image is very light
    - hard-light otherwise
 18.Upscale (double) the Work image, to generate the Final Outcome   
 19.Flatten the image is asked by the user
 
 ----------------------------------------------------------------
 COPYRIGHT NOTICE
 ----------------------------------------------------------------
 
 This program is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation either version 3 of the License, or
 (at your option) any later version.  
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program if not, you can view the GNU General Public
 License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html
 Alternatively you can write to the Free Software Foundation, Inc., 675 Mass
 Ave, Cambridge, MA 02139, USA.
 
 -------------------------------------------------------------------
 other info
 -------------------------------------------------------------------
 
'''

#!/usr/bin/env python
from gimpfu import *

#
# ======================================================
# ANCILLARY MODULE : SET EACH PIXEL BLACK OR TRANSPARENT
# ------------------------------------------------------
def Stretch_RGB_colours (layer) :
    ''' Adjustment for each COLOUR of :
        percentage of pixels with the value 0 
        percentage of pixels with the value 255
    
    Parameters:
    layer : The layer of the image that is selected.
    '''    

    img = layer.image
    
    target = 1  # target percent

    # ============================
    # LEVELS STRETCH FOR COLOURS
    # ============================

    # RED 

    # =========== 
    # ZERO QUOTE
    # ===========  
    
    fromV = 0
    toV = 1
    percent = 0
    while (percent < target) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (layer, HISTOGRAM_RED, fromV, toV)
        percent = percentile * 100  
        toV += 1
    #gimp.message ("percent RED at value 0 = " + str(percent))
    fromMinRED = toV - 1

    
    fromV = 254
    toV = 255
    percent = 0
    while (percent < target) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (layer, HISTOGRAM_RED, fromV, toV)
        percent = percentile * 100  
        fromV -= 1
    #gimp.message ("percent RED at value 255 = " + str(percent))
    toMaxRED = fromV + 1
    
    #gimp.message ("Red From,To = "+str(fromMinRED)+","+str(toMaxRED))
    
    # GREEN 

    # =========== 
    # ZERO QUOTE
    # ===========  
    
    fromV = 0
    toV = 1
    percent = 0
    while (percent < target) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (layer, HISTOGRAM_GREEN, fromV, toV)
        percent = percentile * 100  
        toV += 1
    #gimp.message ("percent GREEN at value 0 = " + str(percent))
    fromMinGREEN = toV - 1

    
    fromV = 254
    toV = 255
    percent = 0
    while (percent < target) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (layer, HISTOGRAM_GREEN, fromV, toV)
        percent = percentile * 100  
        fromV -= 1
    #gimp.message ("percent GREEN at value 255 = " + str(percent))
    toMaxGREEN = fromV + 1
   
    #gimp.message ("Green From,To = "+str(fromMinGREEN)+","+str(toMaxGREEN))
    
    # BLUE 

    # =========== 
    # ZERO QUOTE
    # ===========  
    
    fromV = 0
    toV = 1
    percent = 0
    while (percent < target) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (layer, HISTOGRAM_BLUE, fromV, toV)
        percent = percentile * 100  
        toV += 1
    #gimp.message ("percent BLUE at value 0 = " + str(percent))
    fromMinBLUE = toV - 1

    
    fromV = 254
    toV = 255
    percent = 0
    while (percent < target) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (layer, HISTOGRAM_BLUE, fromV, toV)
        percent = percentile * 100  
        fromV -= 1
    #gimp.message ("percent BLUE at value 255 = " + str(percent))
    toMaxBLUE = fromV + 1
   
    #gimp.message ("Blue From,To = "+str(fromMinBLUE)+","+str(toMaxBLUE))
    
    # APPLY MODIFICATIONS TO LEVELS
    
    pdb.gimp_levels (layer, 1, fromMinRED, toMaxRED, 1, 0, 255)
    pdb.gimp_levels (layer, 2, fromMinRED, toMaxRED, 1, 0, 255)
    pdb.gimp_levels (layer, 3, fromMinRED, toMaxRED, 1, 0, 255)
    
    return(layer)
#
# ======================================================
# ANCILLARY MODULE : SET EACH PIXEL BLACK OR TRANSPARENT
# ------------------------------------------------------
def black_or_transparent(layer) :
    ''' Analysis of each pixel: if has any RGB value > 0, it is converted to pure BLACK
    Otherwise it is converted to pure TRANSPARENCY
    
    Note that this version is fast, since it make use of tiles.
    
    Parameters:
    layer : layer The layer of the image that is selected.
    '''
    
    img = layer.image
    # Indicates that the process has started.
    gimp.progress_init("Transforming " + layer.name + "into a BLACK or TRANSPARENT layer...")

    # Set up an undo group, so the operation will be undone in one step.
    pdb.gimp_image_undo_group_start(img)

    # Get the layer position.
    pos = 0;
    for i in range(len(img.layers)):
        if(img.layers[i] == layer):
            pos = i
    
    # Create a new layer to save the results (otherwise is not possible to undo the operation).
     
    newLayer = gimp.Layer(img,  layer.name+"_work", layer.width, layer.height, layer.type, layer.opacity, layer.mode)
    img.add_layer(newLayer, pos)
    layerName = layer.name
    
    # Clear the new layer.
    pdb.gimp_edit_clear(newLayer)
    newLayer.flush()
    
    # Convert the pixels to BLACK or TRANSPARENT.
    try:
        # Calculate the number of tiles.
        tn = int(layer.width / 64)
        if(layer.width % 64 > 0):
            tn += 1
        tm = int(layer.height / 64)
        if(layer.height % 64 > 0):
            tm += 1
        
        # Iterate over the tiles.
        for i in range(tn):
            for j in range(tm):
                # Update the progress bar.
                gimp.progress_update(float(i*tm + j) / float(tn*tm))
        
                # Get the tiles.
                srcTile = layer.get_tile(False, j, i)
                dstTile = newLayer.get_tile(False, j, i)
        
                # Iterate over the pixels of each tile.
                for x in range(srcTile.ewidth):
                    for y in range(srcTile.eheight):
                        # Get the pixel and calculate his gray value.
                        pixel = srcTile[x,y]
                        red = ord(pixel[0])
                        green = ord(pixel[1])
                        blue = ord(pixel[2])
                        transp = ord(pixel[3])
                        if (red+green+blue) > 0 and transp > 0 :
                            newred = 0
                            newgreen = 0
                            newblue = 0
                            newtransp = 255
                        else :
                            newred = 0
                            newgreen = 0
                            newblue = 0
                            newtransp = 0
    
                        res = chr(newred) + chr(newgreen) + chr(newblue) + chr(newtransp)
                        
                        # If the image has other channels channel) copy his values.
                        if(len(pixel) > 4):
                            for k in range(len(pixel)-4):
                                res += pixel[k+4]
                                
                        # Save the value in the result layer.
                        dstTile[x,y] = res
        
        # Update the new layer.
        newLayer.flush()
        newLayer.merge_shadow(True)
        newLayer.update(0, 0, newLayer.width, newLayer.height)
        
        # Remove the old layer.
        img.remove_layer(layer)
        # Change the name of the new layer (two layers can not have the same name).
        newLayer.name = layerName
     
        
    except Exception as err:
        gimp.message("Unexpected error: " + str(err))
    
    # Close the undo group.
    pdb.gimp_image_undo_group_end(img)
    
    # End progress.
    pdb.gimp_progress_end()

    return (newLayer)
    
#==================================================================================================
# MAIN MODULE
#-------------------------------------------------------------------------------------------------- 
def Mosaic_Lyle_Style(inImage, inLayer, inSize, doFlatten) :

    pdb.gimp_context_push
    currFG = pdb.gimp_context_get_foreground()
    currBG = pdb.gimp_context_get_background()

    # calculate outcome width and height dimensions on the basis of W/H ratio of the input image
    WTRatioF = float (inImage.width * 100.00/ (inImage.width+inImage.height) * 1.00)
    newWidth = int(float(inSize * WTRatioF /100.0))
    newHeight = inSize - newWidth
    #gimp.message ("OUT sizes " +str(newWidth) +"," +str(newHeight))
    newImage = pdb.gimp_image_new(newWidth, newHeight, RGB)
    BG_Layer = pdb.gimp_layer_new_from_drawable(inLayer, newImage)     # background layer
    BG_Layer.name = "Background Layer"
    BG_Layer.mode = NORMAL_MODE
    BG_Layer.opacity = 100.0 
    newImage.add_layer(BG_Layer, 0)
    pdb.gimp_item_transform_scale(BG_Layer, 0, 0, newWidth, newHeight)      # rescale BG
    
    #pdb.gimp_display_new (newImage)
    
    BG_Layer = Stretch_RGB_colours (BG_Layer)   # stretch RGB

    Colour_Isophote_Layer = pdb.gimp_layer_new_from_drawable(BG_Layer, newImage) 
    Colour_Isophote_Layer.name = "Colour Isophote Layer"
    Colour_Isophote_Layer.mode = NORMAL_MODE
    Colour_Isophote_Layer.opacity = 100.0 
    newImage.add_layer(Colour_Isophote_Layer, 0)
    # [G'MIC] Isophotes: -gimp_isophotes 7,5,1,0
    pdb.plug_in_gmic(newImage, Colour_Isophote_Layer, 1,    # 1=input only active layer
        "-v - -gimp_isophotes 7,5,1,0")            # was 7

    Alpha_Isophote_Layer = pdb.gimp_layer_new_from_drawable(BG_Layer, newImage) 
    Alpha_Isophote_Layer.name = "Alpha Isophote Layer"
    Alpha_Isophote_Layer.mode = NORMAL_MODE
    Alpha_Isophote_Layer.opacity = 100.0 
    newImage.add_layer(Alpha_Isophote_Layer, 0)
    # [G'MIC] Isophotes: -gimp_isophotes 7,5,0,0
    pdb.plug_in_gmic(newImage, Colour_Isophote_Layer, 1,    # 1=input only active layer
        "-v - -gimp_isophotes 7,5,0,0")
    Alpha_Isophote_Layer = black_or_transparent(Alpha_Isophote_Layer)

    RobA_Layer = pdb.gimp_layer_new_from_drawable(Alpha_Isophote_Layer, newImage) 
    RobA_Layer.name = "RobA Mosaic Helper Layer"
    RobA_Layer.mode = NORMAL_MODE
    RobA_Layer.opacity = 100.0 
    newImage.add_layer(RobA_Layer, 0)
    pdb.gimp_context_set_sample_transparent (True)
    pdb.gimp_context_set_sample_threshold (0.0)
    pdb.gimp_context_set_sample_merged (False)
    pdb.gimp_image_select_color (newImage, CHANNEL_OP_REPLACE, RobA_Layer, (0,0,0,0))
    
    pdb.script_fu_mosaic_tile_helper (newImage, RobA_Layer, 10, 0, True, False, (0,0,0))
    RobA_Path = pdb.gimp_image_get_active_vectors (newImage)
    pdb.gimp_item_set_name (RobA_Path, "RobA Path")

    # create the brush
    myBrushName = "myDot"
    myDot = pdb.gimp_brush_duplicate ("2. Hardness 100")
    pdb.gimp_brush_rename (myDot, myBrushName)
    pdb.gimp_brush_set_radius (myBrushName, 1.0)
    pdb.gimp_brush_set_spacing (myBrushName, 1000)
    pdb.gimp_context_set_brush (myBrushName)
    pdb.gimp_context_set_foreground ((0,0,0,255))

    # layer with the dots
    Dots_Layer = pdb.gimp_layer_new_from_drawable(RobA_Layer, newImage) 
    Dots_Layer.name = "Dots Layer"
    Dots_Layer.mode = NORMAL_MODE
    Dots_Layer.opacity = 100.0 
    newImage.add_layer(Dots_Layer, 0)
    pdb.gimp_selection_invert (newImage)    # swith from the paths area to the isophote-borders area, to stroke
    pdb.gimp_context_set_brush_size (3.0)
    pdb.gimp_edit_stroke (Dots_Layer)
    
    pdb.gimp_edit_cut (Dots_Layer)
    pdb.gimp_selection_none (newImage)

    pdb.gimp_context_set_brush_size (1.0)
    pdb.gimp_edit_stroke_vectors (Dots_Layer, RobA_Path)
    pdb.gimp_brush_delete (myBrushName)     # no need to keep it
    pdb.gimp_image_select_color (newImage, CHANNEL_OP_REPLACE, Dots_Layer, (0,0,0,0))
    pdb.gimp_edit_fill (Dots_Layer, WHITE_FILL)
    pdb.gimp_selection_none (newImage)
    pdb.gimp_image_remove_vectors (newImage, RobA_Path)
    
    #pdb.gimp_display_new(newImage)
    #return

    Skeleton_Layer = pdb.gimp_layer_new_from_drawable(Dots_Layer, newImage) 
    Skeleton_Layer.name = "Skeleton Layer"
    Skeleton_Layer.mode = NORMAL_MODE
    Skeleton_Layer.opacity = 100.0 
    newImage.add_layer(Skeleton_Layer, 0)
    # [G'MIC] Skeleton: -gimp_skeleton 0,0,0,1,0
    pdb.plug_in_gmic(newImage, Colour_Isophote_Layer, 1,    # 1=input only active layer
        "-v - -gimp_skeleton 0,0,0,1,0")
    
    Contours_Layer = pdb.gimp_layer_new_from_drawable(Alpha_Isophote_Layer, newImage) 
    Contours_Layer.name = "Contours Layer (copy of the Alpha Isophotes)"
    Contours_Layer.mode = NORMAL_MODE
    Contours_Layer.opacity = 100.0 
    newImage.add_layer(Contours_Layer, 0)
    
    Full_Grid_Layer = pdb.gimp_layer_new_from_visible(newImage, newImage, "Full Grid Layer") 
    Full_Grid_Layer.mode = NORMAL_MODE
    Full_Grid_Layer.opacity = 100.0 
    newImage.add_layer(Full_Grid_Layer, 0)
    pdb.gimp_image_select_color (newImage, CHANNEL_OP_REPLACE, Full_Grid_Layer, (0,0,0,255))    # select tiles area (black)
    pdb.gimp_selection_invert (newImage)    # swith from the tiles area to the grid area, to remove from top layers
    
    Colours_Layer = pdb.gimp_layer_new_from_drawable(Colour_Isophote_Layer, newImage) 
    Colours_Layer.name = "Colours Layer"
    Colours_Layer.mode = GRAIN_MERGE_MODE
    Colours_Layer.opacity = 100.0 
    newImage.add_layer(Colours_Layer, 0)
    pdb.gimp_edit_cut (Colours_Layer)

    Colours_Layer2 = pdb.gimp_layer_new_from_drawable(Colours_Layer, newImage) 
    Colours_Layer2.name = "2. Colours Layer"
    Colours_Layer2.mode = HARDLIGHT_MODE
    Colours_Layer2.opacity = 90.0 
    newImage.add_layer(Colours_Layer2, 0)
    
    Original_Layer = pdb.gimp_layer_new_from_drawable(BG_Layer, newImage)
    Original_Layer.name = "Copy of original drawable"
    Original_Layer.mode = SCREEN_MODE
    Original_Layer.opacity = 50.0 
    newImage.add_layer(Original_Layer, 0)
    pdb.gimp_edit_cut (Original_Layer)
    
    #pdb.gimp_display_new(newImage) 
    #return()
    
    Original_Layer2 = pdb.gimp_layer_new_from_drawable(BG_Layer, newImage)
    Original_Layer2.name = "2. Copy of original drawable"
    Original_Layer2.mode = SCREEN_MODE
    Original_Layer2.opacity = 50.0 
    newImage.add_layer(Original_Layer2, 0)
    pdb.gimp_edit_cut (Original_Layer2)
    
    pdb.gimp_context_set_background ((10,20,30,255))
    pdb.gimp_edit_fill (Full_Grid_Layer, BACKGROUND_FILL)

    pdb.gimp_selection_invert (newImage)    # swith to return to the tiles area
    pdb.script_fu_add_bevel (newImage, Colours_Layer, 2, False, False)
    pdb.script_fu_add_bevel (newImage, Colours_Layer2, 1, False, False)
    pdb.script_fu_add_bevel (newImage, Original_Layer, 1, False, False)
    pdb.script_fu_add_bevel (newImage, Original_Layer2, 1, False, False)
    
    pdb.gimp_selection_none (newImage)

    # decide which Mode and Opacity depending on the MeanValue of Top layer
    (meanValue, stdDev, median, pix, cnt, perc) = pdb.gimp_histogram (BG_Layer, HISTOGRAM_VALUE, 0 ,255)
    #gimp.message ("meanValue = "+str(meanValue))
    if meanValue < 107 :    # dark  
        pdb.gimp_layer_set_mode (Original_Layer2, SCREEN_MODE)
        pdb.gimp_layer_set_opacity (Original_Layer2, 70.0)
    elif meanValue > 203 :    # light  
        pdb.gimp_layer_set_mode (Original_Layer2, BURN_MODE)
        pdb.gimp_layer_set_opacity (Original_Layer2, 80.0)
    else :  # average      
        pdb.gimp_layer_set_mode (Original_Layer2, HARDLIGHT_MODE)
        pdb.gimp_layer_set_opacity (Original_Layer2, 80.0)

    # rescale
    pdb.gimp_image_scale (newImage, 2*newWidth, 2*newHeight)
        
    if (doFlatten == True) :
        pdb.gimp_image_flatten(newImage)

    pdb.gimp_context_pop
    pdb.gimp_context_set_foreground(currFG)
    pdb.gimp_context_set_background(currBG)
 
    
    pdb.gimp_display_new(newImage) 
    
    
    return
    



register(
    "Mosaic_Lyle_Style",
    "make a Mosaic out of an image",
    "transform an input image into a Mosaic",
    "JFM-DN",
    "Open source (BSD 3-clause license)",
    "2016",
    "Mosaic Lyle Style",
    "RGB*",
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "drawable", "Input drawable", None),
      (PF_ADJUSTMENT, "dimension", "Work image dimension", 1800, (600, 3000, 100)),
      (PF_TOGGLE, "flatten", "Flatten Image?", False),

    ],
    [],
    Mosaic_Lyle_Style,
    menu="<Image>/Diego/Artistic"
    )
main()
