'''
 Filter to implement the creation of a MOSAIC STAINED GLASS
 ............................................................. 
 
 Release 1.0   initial
 
 
Flow implemented to get the final result:     
 
 1. Create the work image :
    # on the basis of the input image size and the desired tile size
    # the tile size is created almost fixed 10*10 pixels in the work image
    # thus the work image size is reduced according to the desired tile size / 10
 2. Stretch the colours using an ancillary function (embedded)
 3. Remove too dark pixels (0 set to 16) to have always enough contrast with the grid
 4. Add a Layer (duplicate BG) and perform some preprocessing:
    # a large blur (24, 48)
    # the cartoonized plug-in with removal of small areas
    # the G'MIC Inpaint Holes with additional removal of small areas
 5. Add a Layer (duplicate preprocessed layer) and run G'MIC Isophotes with flat colours as BG
 6. Add a Layer (duplicate preprocessed layer) and run G'MIC Isophotes with transparent BG
    and filter to have only black or transparent pixels using an ancillary function (embedded)
 7. Add a new Layer and fill it with a yellow colour
 8. Prepare the array of present colours using an ancillary function (embedded)
 9. Create a work brush of 1 pixel with max specing (1000 which corresponds to 10 pixels)
 10.Using the Colour Isophotes Layer as base, loop on the array of colours and for each one:
    # convert the selection to a path and stroke it to have the contours
    # aspply G'MIC Cracks on the working yellow layer to get the cracks on the specific selected area
 11.Remove the yellow bg from the fully cracked layer and fill the grid with a dark bluish colour   
 12.Add Layers for colour embellishment:
    # a layer filled with a plasma (overlay)
    # a layer filled with a gradient (overlay)
    # a layer filled with G'MIC Bokeh (hardlight)
    # a layer filled with a sky gradient (dissolve), setting the light origin where selected by the user
 13.Apply a bevel effect on the tiles in the Crack Layer 12.Create a new layer combining the whole grid
 14.Reduce opacity of the Colour Isophote Layer
 15.Create a new layer and add a border
 16.Apply to the border the Metal pattern selected by the user
 17.Upscale the Work image, to generate the Final Outcome   
 18.Flatten the image is asked by the user
 
 ----------------------------------------------------------------
 COPYRIGHT NOTICE
 ----------------------------------------------------------------
 
 This program is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation either version 3 of the License, or
 (at your option) any later version.  
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program if not, you can view the GNU General Public
 License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html
 Alternatively you can write to the Free Software Foundation, Inc., 675 Mass
 Ave, Cambridge, MA 02139, USA.
 
 -------------------------------------------------------------------
 other info
 -------------------------------------------------------------------
 
'''

#!/usr/bin/env python
from gimpfu import *

#
# ======================================================
# ANCILLARY MODULE : SET EACH PIXEL BLACK OR TRANSPARENT
# ------------------------------------------------------
def Stretch_RGB_colours (layer) :
    ''' Adjustment for each COLOUR of :
        percentage of pixels with the value 0 
        percentage of pixels with the value 255
    
    Parameters:
    layer : The layer of the image that is selected.
    '''    

    img = layer.image
    
    target = 1  # target percent

    # ============================
    # LEVELS STRETCH FOR COLOURS
    # ============================

    # RED 

    # =========== 
    # ZERO QUOTE
    # ===========  
    
    fromV = 0
    toV = 1
    percent = 0
    while (percent < target) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (layer, HISTOGRAM_RED, fromV, toV)
        percent = percentile * 100  
        toV += 1
    #gimp.message ("percent RED at value 0 = " + str(percent))
    fromMinRED = toV - 1

    
    fromV = 254
    toV = 255
    percent = 0
    while (percent < target) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (layer, HISTOGRAM_RED, fromV, toV)
        percent = percentile * 100  
        fromV -= 1
    #gimp.message ("percent RED at value 255 = " + str(percent))
    toMaxRED = fromV + 1
    
    # GREEN 

    # =========== 
    # ZERO QUOTE
    # ===========  
    
    fromV = 0
    toV = 1
    percent = 0
    while (percent < target) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (layer, HISTOGRAM_GREEN, fromV, toV)
        percent = percentile * 100  
        toV += 1
    #gimp.message ("percent GREEN at value 0 = " + str(percent))
    fromMinGREEN = toV - 1

    
    fromV = 254
    toV = 255
    percent = 0
    while (percent < target) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (layer, HISTOGRAM_GREEN, fromV, toV)
        percent = percentile * 100  
        fromV -= 1
    #gimp.message ("percent GREEN at value 255 = " + str(percent))
    toMaxGREEN = fromV + 1
   
    #gimp.message ("Green From,To = "+str(fromMinGREEN)+","+str(toMaxGREEN))
    
    # BLUE 

    # =========== 
    # ZERO QUOTE
    # ===========  
    
    fromV = 0
    toV = 1
    percent = 0
    while (percent < target) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (layer, HISTOGRAM_BLUE, fromV, toV)
        percent = percentile * 100  
        toV += 1
    #gimp.message ("percent BLUE at value 0 = " + str(percent))
    fromMinBLUE = toV - 1

    
    fromV = 254
    toV = 255
    percent = 0
    while (percent < target) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (layer, HISTOGRAM_BLUE, fromV, toV)
        percent = percentile * 100  
        fromV -= 1
    #gimp.message ("percent BLUE at value 255 = " + str(percent))
    toMaxBLUE = fromV + 1
   
    #gimp.message ("Blue From,To = "+str(fromMinBLUE)+","+str(toMaxBLUE))
    
    # APPLY MODIFICATIONS TO LEVELS
    
    pdb.gimp_levels (layer, 1, fromMinRED, toMaxRED, 1, 0, 255)
    pdb.gimp_levels (layer, 2, fromMinRED, toMaxRED, 1, 0, 255)
    pdb.gimp_levels (layer, 3, fromMinRED, toMaxRED, 1, 0, 255)
    
    return(layer)
#
# ======================================================
# ANCILLARY MODULE : SET EACH PIXEL BLACK OR TRANSPARENT
# ------------------------------------------------------
def black_or_transparent(layer) :
    ''' Analysis of each pixel: if has any RGB value > 0, it is converted to pure BLACK
    Otherwise it is converted to pure TRANSPARENCY
    
    Note that this version is fast, since it make use of tiles.
    
    Parameters:
    layer : layer The layer of the image that is selected.
    '''
    
    img = layer.image
    # Indicates that the process has started.
    gimp.progress_init("Transforming " + layer.name + "into a BLACK or TRANSPARENT layer...")

    # Get the layer position.
    pos = 0;
    for i in range(len(img.layers)):
        if(img.layers[i] == layer):
            pos = i
    
    # Create a new layer to save the results (otherwise is not possible to undo the operation).
     
    newLayer = gimp.Layer(img,  layer.name+"_work", layer.width, layer.height, layer.type, layer.opacity, layer.mode)
    img.add_layer(newLayer, pos)
    layerName = layer.name
    
    # Clear the new layer.
    pdb.gimp_edit_clear(newLayer)
    newLayer.flush()
    
    # Convert the pixels to BLACK or TRANSPARENT.
    try:
        # Calculate the number of tiles.
        tn = int(layer.width / 64)
        if(layer.width % 64 > 0):
            tn += 1
        tm = int(layer.height / 64)
        if(layer.height % 64 > 0):
            tm += 1
        
        # Iterate over the tiles.
        for i in range(tn):
            for j in range(tm):
                # Update the progress bar.
                gimp.progress_update(float(i*tm + j) / float(tn*tm))
        
                # Get the tiles.
                srcTile = layer.get_tile(False, j, i)
                dstTile = newLayer.get_tile(False, j, i)
        
                # Iterate over the pixels of each tile.
                for x in range(srcTile.ewidth):
                    for y in range(srcTile.eheight):
                        # Get the pixel and calculate his gray value.
                        pixel = srcTile[x,y]
                        red = ord(pixel[0])
                        green = ord(pixel[1])
                        blue = ord(pixel[2])
                        transp = ord(pixel[3])
                        if (red+green+blue) > 0 and transp > 0 :
                            newred = 0
                            newgreen = 0
                            newblue = 0
                            newtransp = 255
                        else :
                            newred = 0
                            newgreen = 0
                            newblue = 0
                            newtransp = 0
    
                        res = chr(newred) + chr(newgreen) + chr(newblue) + chr(newtransp)
                        
                        # If the image has other channels channel) copy his values.
                        if(len(pixel) > 4):
                            for k in range(len(pixel)-4):
                                res += pixel[k+4]
                                
                        # Save the value in the result layer.
                        dstTile[x,y] = res
        
        # Update the new layer.
        newLayer.flush()
        newLayer.merge_shadow(True)
        newLayer.update(0, 0, newLayer.width, newLayer.height)
        
        # Remove the old layer.
        img.remove_layer(layer)
        # Change the name of the new layer (two layers can not have the same name).
        newLayer.name = layerName
     
        
    except Exception as err:
        gimp.message("Unexpected error: " + str(err))
    
    # End progress.
    pdb.gimp_progress_end()

    return (newLayer)
#
# ======================================================
# ANCILLARY MODULE : SET EACH PIXEL WITH ALPHA = 255
# ------------------------------------------------------
def alpha_to_255(layer) :
    ''' Analysis of each pixel: set alpha = 255 and adjust Yellow tint
    the "collateral effects" of G'MIC filter Cracks are:
    - the base color (yellow) altered both in tint and alpha
    but we need to select the pure yellow tint to remove the base
    
    Note that this version makes use of tiles.
    
    Parameters:
    layer : layer The layer of the image that is selected.
    '''
    
    img = layer.image
    # Indicates that the process has started.
    gimp.progress_init("Transforming " + layer.name + " setting alpha=255...")

    # Get the layer position.
    pos = 0;
    for i in range(len(img.layers)):
        if(img.layers[i] == layer):
            pos = i
    
    # Create a new layer to save the results (otherwise is not possible to undo the operation).
     
    newLayer = gimp.Layer(img,  layer.name+"_work", layer.width, layer.height, layer.type, layer.opacity, layer.mode)
    img.add_layer(newLayer, pos)
    layerName = layer.name
    
    # Clear the new layer.
    pdb.gimp_edit_clear(newLayer)
    newLayer.flush()
    
    # Convert the pixels to BLACK or TRANSPARENT.
    try:
        # Calculate the number of tiles.
        tn = int(layer.width / 64)
        if(layer.width % 64 > 0):
            tn += 1
        tm = int(layer.height / 64)
        if(layer.height % 64 > 0):
            tm += 1
        
        # Iterate over the tiles.
        for i in range(tn):
            for j in range(tm):
                # Update the progress bar.
                gimp.progress_update(float(i*tm + j) / float(tn*tm))
        
                # Get the tiles.
                srcTile = layer.get_tile(False, j, i)
                dstTile = newLayer.get_tile(False, j, i)
        
                # Iterate over the pixels of each tile.
                for x in range(srcTile.ewidth):
                    for y in range(srcTile.eheight):
                        # Get the pixel and set alpha to 255.
                        pixel = srcTile[x,y]
                        red = ord(pixel[0])
                        green = ord(pixel[1])
                        blue = ord(pixel[2])
                        transp = ord(pixel[3])
                        # sometimes the yellow base is changed by G'MIC in tint and alpha
                        if (red+green)>360:
                            newred = 255
                            newgreen = 255
                        else:
                            newred = 0
                            newgreen = 0
                        
                        newtransp = 255
    
                        res = chr(newred) + chr(newgreen) + chr(blue) + chr(newtransp)
                        
                        # If the image has other channels channel) copy his values.
                        if(len(pixel) > 4):
                            for k in range(len(pixel)-4):
                                res += pixel[k+4]
                                
                        # Save the value in the result layer.
                        dstTile[x,y] = res
        
        # Update the new layer.
        newLayer.flush()
        newLayer.merge_shadow(True)
        newLayer.update(0, 0, newLayer.width, newLayer.height)
        
        # Remove the old layer.
        img.remove_layer(layer)
        # Change the name of the new layer (two layers can not have the same name).
        newLayer.name = layerName
     
        
    except Exception as err:
        gimp.message("Unexpected error: " + str(err))
    
    # End progress.
    pdb.gimp_progress_end()

    return (newLayer)
#
# =======================================================
# ANCILLARY MODULE : GET ARRAY OF IMAGE COLOURS (max=256)
# -------------------------------------------------------
def Get_array_of_colours(layer, nrc):
    ''' Analysis of each pixel, find its colour, create an array of all detected colours.
    If more than 256 return an error.
    
    Note that this version makes use of tiles.
    
    Parameters:
    layer : layer The layer of the image that is selected.
    '''
    # Indicates that the process has started.
    gimp.progress_init("Analysing " + layer.name + "to create the array of colours...")

    # Get Image ID from Layer
    img = layer.image
    
    # please take note that the following double conversion is needed here
    # if performed in the caller something goes wrong in here
    
    #####################################
    # forced nr of colours thru Indexed #
    #-----------------------------------#
    pdb.gimp_image_convert_indexed (img, NO_DITHER, MAKE_PALETTE,
                                    nrc, False, False, "simply ignored")
    is_indexed = pdb.gimp_drawable_is_indexed (layer)
    if is_indexed == False:
        gimp.message ("conversion to indexed failed")
        pdb.gimp_display_new(img)
        return
    
    pdb.gimp_image_convert_rgb (img)       # once forced nr of colours back to RGB

    
    # Analyse each pixel.
    try:
    # Calculate the number of tiles.
        tn = int(layer.width / 64)
        if(layer.width % 64 > 0):
            tn += 1
        tm = int(layer.height / 64)
        if(layer.height % 64 > 0):
            tm += 1
        
        # Create an ARRAY.
        colours = []
        ic = 0
        retmsg = "nothing done"
        
        # Iterate over the tiles.
        for i in range(tn):
            for j in range(tm):
                # Update the progress bar.
                gimp.progress_update(float(i*tm + j) / float(tn*tm))
        
                # Get the tiles.
                srcTile = layer.get_tile(False, j, i)
        
                # Iterate over the pixels of each tile.
                for x in range(srcTile.ewidth):
                    for y in range(srcTile.eheight):
                        # Get the pixel and find its colour.
                        pixel = srcTile[x,y]
                        red = ord(pixel[0])
                        green = ord(pixel[1])
                        blue = ord(pixel[2])
                        new = 1
                        ioc = 0
                        #gimp.message (str(x)+","+str(y))
                        if ic == 0:
                            #gimp.message ("first colour")
                            colours.append(red)
                            colours.append(green)
                            colours.append(blue)
                            #gimp.message(str(colours[ic]))
                            ic += 3
                        else:    
                            for ioc in range(0, ic, 3):
                                #gimp.message ("next colour")
                                if (red == colours[ioc+0]
                                    and green == colours[ioc+1]
                                    and blue == colours[ioc+2]): 
                                    new = 0
                            if new == 1:
                                #gimp.message ("new colour")
                                colours.append(red)
                                colours.append(green)
                                colours.append(blue)
                                #gimp.message(str(colours[ic]))
                                ic += 3
                        if ic/3 > 256:
                            retmsg = "More than 256 colours" 
     
        retmsg = "OK, colours found = "+str(ic/3)
        
    except Exception as err:
        gimp.message("Unexpected error: " + str(err))
    
    # End progress.
    pdb.gimp_progress_end()
    
    #gimp.message(retmsg)
    
    return(retmsg, colours)

#==================================================================================================
#
#                                   MAIN MODULE
#
#--------------------------------------------------------------------------------------------------

def Mosaic_Stained_Glass (inImage, inLayer,
                          glassPattern, metalPattern,
                          skyGradient,  lightOrigin,
                          inNrCol, inTileSize,
                          doFlatten) :

    pdb.gimp_context_push
    currFG = pdb.gimp_context_get_foreground()
    currBG = pdb.gimp_context_get_background()
    
    originalname = pdb.gimp_item_get_name (inLayer)
    # calculate work image width and height dimensions
    # on the basis of the input image and the desired tile size
    # the tile size is almost fixed 10*10 pixels in the work image
    # thus the work image size is reduced according to the desired tile size / 10
    
    divisor = inTileSize / 10.00
    newWidth = int(float(inImage.width / divisor))
    newHeight = int(float(inImage.height / divisor))
    # but enlarge too small pictures
    if newWidth+newHeight < 2000:
        newWidth *= 2
        newHeight *= 2

    #gimp.message ("OUT sizes " +str(newWidth) +"," +str(newHeight))
    
    newImage = pdb.gimp_image_new(newWidth, newHeight, RGB)
    BG_Layer = pdb.gimp_layer_new_from_drawable(inLayer, newImage)     # background layer
    BG_Layer.name = "Background Layer"
    BG_Layer.mode = NORMAL_MODE
    BG_Layer.opacity = 100.0 
    newImage.add_layer(BG_Layer, 0)
    pdb.gimp_item_transform_scale(BG_Layer, 0, 0, newWidth, newHeight)      # rescale BG
    
    BG_Layer = Stretch_RGB_colours (BG_Layer)   # stretch RGB before Isophotes
    
    # but shift too dark nuances
     
    lowestValue=16
    newCentre=0.60
    pdb.gimp_levels(BG_Layer, 0, 0, 255, newCentre, lowestValue, 255)
    
    pdb.gimp_edit_named_copy (BG_Layer, "original:"+originalname)
    
    # pre-processing
    # --------------
    Preprocessed_Layer = pdb.gimp_layer_new_from_drawable(BG_Layer, newImage) 
    Preprocessed_Layer.name = "Preprocessed Layer"
    Preprocessed_Layer.mode = NORMAL_MODE
    Preprocessed_Layer.opacity = 100.0 
    newImage.add_layer(Preprocessed_Layer, 0)
    # 1.preprocess: sel-blur
    pdb.plug_in_sel_gauss (newImage, Preprocessed_Layer, 24, 48)
    # 2.preprocess: cartonizer with removal of small areas
    pdb.plug_in_cartoonizer(newImage, Preprocessed_Layer,
                # nr-of-colors, colorweight, weightingmode, algorithm
                inNrCol, 8, 2, 0,
                # B&W, legend, denoise 
                False, False, True,
                # spotsize, despotting-iterat, erode, erode-iterat
                100, 5, True, 20,        
                # dolines, color, removeshorter, postprocess
                False, 0, 0, 0, 0)
    # 3.preprocess: G'MIC Inpaint Holes with additional removal of small areas
        # + [G'MIC] Inpaint [holes]: -gimp_inpaint_holes 20,20,1
    pdb.plug_in_gmic(newImage, Preprocessed_Layer, 1,    # 1=input only active layer
        "-v - -gimp_inpaint_holes "
        +str(100)
        +",20,1")
    
    
    Colour_Isophote_Layer = pdb.gimp_layer_new_from_drawable(Preprocessed_Layer, newImage) 
    Colour_Isophote_Layer.name = "Colour Isophote Layer"
    Colour_Isophote_Layer.mode = NORMAL_MODE
    Colour_Isophote_Layer.opacity = 100.0 
    newImage.add_layer(Colour_Isophote_Layer, 0)
    # [G'MIC] Isophotes: -gimp_isophotes 7,5,1,0
    pdb.plug_in_gmic(newImage, Colour_Isophote_Layer, 1,    # 1=input only active layer
        "-v - -gimp_isophotes "+
        str(inNrCol)
        +",2.5,1,0")            
    
    Alpha_Isophote_Layer = pdb.gimp_layer_new_from_drawable(Preprocessed_Layer, newImage) 
    Alpha_Isophote_Layer.name = "Alpha Isophote Layer"
    Alpha_Isophote_Layer.mode = NORMAL_MODE
    Alpha_Isophote_Layer.opacity = 100.0 
    newImage.add_layer(Alpha_Isophote_Layer, 0)
    # [G'MIC] Isophotes: -gimp_isophotes 7,5,0,0
    pdb.plug_in_gmic(newImage, Colour_Isophote_Layer, 1,    # 1=input only active layer
        "-v - -gimp_isophotes "+
        str(inNrCol)
        +",2.5,0,0")            
    Alpha_Isophote_Layer = black_or_transparent(Alpha_Isophote_Layer)
    
    #######################################################################################
    # code to fill each separate colour area with cracks
    #--------------------------------------------------------------------------------------
    
    Cracks_Layer = pdb.gimp_layer_new(newImage, newWidth, newHeight, RGBA_IMAGE,
                                      "Cracks Layer", 100.0, NORMAL_MODE)
    newImage.add_layer(Cracks_Layer, 0)
    pdb.gimp_context_set_foreground ((255,255,0,255))
    pdb.gimp_edit_fill (Cracks_Layer, FOREGROUND_FILL)
    
    # create an array of the colours    
    (retmsg, colours) = Get_array_of_colours (Colour_Isophote_Layer, inNrCol)
    
    gimp.message (retmsg)
    nr_of_colours = len(colours)/3
    gimp.message ("colours in the array = "+str(nr_of_colours))
    
    # work on each separate colour area
    pdb.gimp_context_set_brush ("2. Hardness 100")
    pdb.gimp_context_set_brush_angle (0.00)
    pdb.gimp_context_set_brush_aspect_ratio (0.00)
    pdb.gimp_context_set_foreground ((0,0,0,255))
    pdb.gimp_context_set_brush_size (1.0)
    ic = 0
    for ic in range (0, len(colours), 3):
        colour_to_select = (colours[ic+0], colours[ic+1], colours[ic+2])
        #gimp.message ("colour nr. "+str(ic/3)+" R,G,B ="+str(colours[ic+0])
        #              +","+str(colours[ic+1])+","+str(colours[ic+1]))
        pdb.gimp_image_select_color (newImage, CHANNEL_OP_REPLACE, Colour_Isophote_Layer, colour_to_select)
        # try
        if pdb.gimp_selection_is_empty (newImage) == True:
            gimp.message ("selection empty for colour nr. "+str(ic/3))
        else:
            pdb.plug_in_sel2path (newImage, Cracks_Layer)
            This_Path = pdb.gimp_image_get_active_vectors (newImage)
            pdb.gimp_edit_stroke_vectors (Cracks_Layer, This_Path)
            #gimp.message ("cracking area selected for colour nr. "+str(ic/3))
            # [G'MIC] Cracks: -gimp_cracks 0.2,-80,0,0,0
            pdb.plug_in_gmic(newImage, Cracks_Layer, 1,    # 1=input only active layer
             "-v - -gimp_cracks 0.05,-80,0,0,0")
    
    # remove the yellow bg and fill the grid with a dark bluish colour
    Cracks_Layer = alpha_to_255 (Cracks_Layer)
    pdb.gimp_image_select_color (newImage, CHANNEL_OP_REPLACE, Cracks_Layer, (255,255,0,255))
    pdb.gimp_edit_cut (Cracks_Layer)
    pdb.gimp_selection_invert (newImage)
    pdb.gimp_context_set_background ((10,20,30,255))
    pdb.gimp_selection_grow (newImage, 1)
    pdb.gimp_edit_fill (Cracks_Layer, BACKGROUND_FILL)
    
    grid_selection = pdb.gimp_selection_save (newImage)
    
    pdb.gimp_selection_invert (newImage)    # swith to return to the tiles area
    
    # embellishment of lights and colour
    # ----------------------------------
    # plasma layer
    PlasmaLayer = pdb.gimp_layer_new(newImage, newWidth, newHeight, RGBA_IMAGE,
                                        "Plasma Layer", 55.0, OVERLAY_MODE)
    newImage.add_layer(PlasmaLayer, 0)
    PlasmaLayer = newImage.active_layer
    pdb.plug_in_plasma(newImage, PlasmaLayer, 17, 3.3)
    
    # gradient layer
    PatternLayer = pdb.gimp_layer_new(newImage, newWidth, newHeight, RGBA_IMAGE,
                                        "Pattern Layer", 55.0, OVERLAY_MODE)
    newImage.add_layer(PatternLayer, 0)
    PatternLayer = newImage.active_layer
 
    pdb.gimp_context_set_pattern(glassPattern)
    pdb.gimp_edit_fill(PatternLayer,PATTERN_FILL)
        
    BokehLayer = pdb.gimp_layer_new_from_visible(newImage, newImage, "Bokeh Layer")
    BokehLayer.opacity = 100.00
    BokehLayer.mode = NORMAL_MODE
    newImage.add_layer(BokehLayer, 0)
    BokehLayer = newImage.active_layer
    # WARNING! the Bokeh filter works ONLY if the underlying layer is in NORMAL_MODE and 100.0% OPACITY !!!
    # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    pdb.plug_in_gmic(newImage, BokehLayer, 1,    # 1=input only active layer
                  #[G'MIC] Bokeh: -gimp_bokeh 3,8,0,30,8,4,0.3,0.2,210,210,80,160,0.7,30,20,20,1,2,170,130,20,110,0.15,0
                  "-v - -gimp_bokeh 3,8,0,50,8,4,0.3,0.2,247,120,29,200,0.7,5,18,9,1,2,233,230,11,200,0.15,0")
    BokehLayer = newImage.active_layer
    # NOW we set opacity and mode as we like
    BokehLayer.opacity = 18.00
    BokehLayer.mode = HARDLIGHT_MODE
    
    SkyLayer=pdb.gimp_layer_new(newImage, newWidth, newHeight, RGBA_IMAGE,
                                   "Sky Layer", 20.00, DISSOLVE_MODE)
    newImage.add_layer(SkyLayer, 0)
    SkyLayer = newImage.active_layer
    
    # sky
    if (lightOrigin==0):    # outside
            fromX = (newWidth/2)
            fromY = 0 
            toX = (newWidth/2)
            toY = newHeight 
            gradType = 0          # gradient-linear
    elif (lightOrigin==1):    # top left
           fromX = 0 
           fromY = 0 
           toX = newWidth 
           toY = newHeight 
           gradType = 2          # gradient-radial
    elif (lightOrigin==2):   # top centre
           fromX = (newWidth/2) 
           fromY = 0 
           toX = (newWidth/2) 
           toY = newHeight 
           gradType = 2          # gradient-radial
    elif (lightOrigin==3):   # top right
           fromX = newWidth  
           fromY = 0 
           toX = 0 
           toY = newHeight 
           gradType = 2          # gradient-radial
    elif (lightOrigin==4):   # centre left
           fromX = 0  
           fromY = (newHeight/2) 
           toX = newWidth 
           toY = (newHeight/2)
           gradType = 2          # gradient-radial
    elif (lightOrigin==5):   # centre centre
           fromX = (newWidth/2)  
           fromY = (newHeight/2) 
           toX = (newWidth/2) 
           toY = newHeight
           gradType = 2          # gradient-radial
    elif (lightOrigin==6):   # centre right
           fromX = newWidth  
           fromY = (newHeight/2) 
           toX = 0 
           toY = (newHeight/2)
           gradType = 2          # gradient-radial
    elif (lightOrigin==7):   # bottom left
           fromX = 0  
           fromY = newHeight  
           toX = newWidth
           toY = 0
           gradType = 2          # gradient-radial
    elif (lightOrigin==8):   # bottom centre
           fromX = (newWidth/2)  
           fromY = newHeight  
           toX = (newWidth/2)
           toY = 0
           gradType = 2          # gradient-radial
    elif (lightOrigin==9):   # bottom right
           fromX = newWidth 
           fromY = newHeight  
           toX = 0
           toY = 0
           gradType = 2          # gradient-radial
             
    pdb.gimp_context_set_gradient(skyGradient)
    pdb.gimp_edit_blend(SkyLayer, 
        3, 				    	 # CUSTOM 
        0,                       # NORMAL MODE
        gradType, 				 # gradient type
        100,                     # opacity
        0,                       # offset
        0,                       # REPEAT-NONE
        True,                    # reverse
        False,                   # no supersampling
        0,                       # supers recursions
        0,                       # supers threshold
        False,                   # no dithering
        fromX,					 # x1 
        fromY,       			 # y1 
        toX,				     # x2
        toY)				     # y2


    
    pdb.gimp_image_select_item (newImage, CHANNEL_OP_REPLACE, grid_selection)

    # experiment bevel
    pdb.script_fu_ml_bevel_reflect_logo_alpha(newImage, Cracks_Layer,                                      # BRL Tiles
                             0,                     #bevel slope option,  0=Flat 2=Round
                             8,                     #flat bevel smoothing,
                             10.0, 7.0, 20.0,       #flat bevel depth, round1 bevel depth, rounf2 bevel depth,
                             1.00,                  #shiny,
                             0, 0,                  #contour, contour depth,
                             1,                     #edge smoothing,
                             None,                  #emap,  <==========
                             0,                     #emap type,
                             0, 4, 0,               #blend mode, blend direction, blend shape,
                             "Metallic Something",          #gradient,
                             315,                   #light direction,
                             0.35,                  #specular,
                             0,                     #detail,
                             1.2, 2.6,              #horiz texture size, vertic texture size,
                             300,                   #emap final size,
                             0,                     #emap final blur,
                             True,                  #autostretch,
                             True,                  #noise,
                             True)                  #keep)
    brl1 = pdb.gimp_image_get_layer_by_name (newImage, "BR Logo")
    pdb.gimp_item_set_visible (brl1, True)
    brl1.name = "BRL 1"
    brlc1 = pdb.gimp_image_get_layer_by_name (newImage, "BR Logo Copy")
    pdb.gimp_item_set_visible (brlc1, True)
    brlc1.name = "BRL Copy 1"
    
    pdb.gimp_selection_none (newImage)
    # reduce opacity of the cartoonised layer to let original transpare below
    Colour_Isophote_Layer.opacity = 50.0
    Colour_Isophote_Layer.mode = MULTIPLY_MODE
    
    pdb.gimp_item_set_visible (Alpha_Isophote_Layer, False)
    # decide which Mode and Opacity depending on the MeanValue of Original
    (meanValue, stdDev, median, pix, cnt, perc) = pdb.gimp_histogram (BG_Layer, HISTOGRAM_VALUE, 0 ,255)
    #gimp.message ("meanValue = "+str(meanValue))
    if meanValue < 107 :    # dark  
        pdb.gimp_layer_set_mode (Preprocessed_Layer, SCREEN_MODE)
        pdb.gimp_layer_set_opacity (Preprocessed_Layer, 70.0)
    elif meanValue > 203 :    # light  
        pdb.gimp_layer_set_mode (Preprocessed_Layer, BURN_MODE)
        pdb.gimp_layer_set_opacity (Preprocessed_Layer, 80.0)
    else :  # average      
        pdb.gimp_layer_set_mode (Preprocessed_Layer, HARDLIGHT_MODE)
        pdb.gimp_layer_set_opacity (Preprocessed_Layer, 80.0)
    #pdb.gimp_item_set_visible (Preprocessed_Layer, False)
    
    TopLayer = newImage.layers[0]  
    BorderColour=(2,2,4) 
    pdb.script_fu_addborder(newImage, TopLayer,
                int((newWidth+newHeight)/20), int((newWidth+newHeight)/20), BorderColour, 20)
    BorderLayer = pdb.gimp_image_get_active_layer (newImage)
    pdb.gimp_context_set_sample_threshold_int (20)
    pdb.gimp_image_select_color (newImage, CHANNEL_OP_REPLACE, BorderLayer, BorderColour)
    pdb.gimp_selection_shrink (newImage, int((newWidth+newHeight)/120))
    pdb.gimp_context_set_pattern (metalPattern)
    pdb.gimp_edit_fill(BorderLayer, PATTERN_FILL)
    
    # rescale
    if inTileSize > 10:
        pdb.gimp_image_scale (newImage, newWidth*divisor, newHeight*divisor)
        
    if (doFlatten == True) :
        pdb.gimp_image_flatten(newImage)

    pdb.gimp_context_pop
    pdb.gimp_context_set_foreground(currFG)
    pdb.gimp_context_set_background(currBG)
    
    pdb.gimp_display_new(newImage) 
    
    return

register(
    "Mosaic_Stained_Glass",
    "to create a mosaic with stained glass from an image",   
    "This script takes an image and transform it into a stained glass mosaic",
    "Diego", 
    "Diego Nassetti ", 
    "March 2016",
    "Mosaic Stained Glass",
    "RGB*",
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "drawable", "Input drawable", None),
      (PF_PATTERN, "glass_pattern", "Glass pattern", "Gold 322"),
      (PF_PATTERN, "metal_pattern", "Metal pattern", "Glasstic_8"),
      (PF_GRADIENT, "sky_gradient", "Sky gradient (light on the right)", "Neon Yellow"),
      (PF_OPTION, "light_origin", "Light origin", 1, ("none","Top left","Top centre","Top right",
                                                      "Centre left","Centre centre","Centre sight",
                                                      "Bottom left","Bottom centre","Bottom right")),
      (PF_ADJUSTMENT, "colours", "Nr of colours", 7, (5, 16, 1)),
      (PF_ADJUSTMENT, "tilesize", "Final tile dimension", 15, (10, 50, 5)),
      (PF_TOGGLE, "flatten", "Flatten Image?", False),

    ],
    [],
    Mosaic_Stained_Glass,
    menu="<Image>/Diego/Artistic"
    )
main()
