'''

 Filter to render the rain effect on the input drawable
 ....................................................................
 
 Release 1.0   initial
 
 Flow implemented to get the final result:      ...detail the major steps
 
 1. to get the best rain distribution, the original image is resized
    scale :
           rescaling smallest dimension to 1000
           rescaling largest dimension proportionally
 2. new drawable (ADDITION, 95%)
 3. fill it with black
 4. apply HSV noise
 5. apply a threshold
 6. apply a motion blur (length = 15, angle = 75)
 7. increase brightness and contrast
 8. black to alpha
 9. new drawable (ADDITION, 80%)
 10.fill it with black
 11.apply HSV noise
 12.apply a threshold
 13.apply a motion blur (length = 25, angle = 79)
 14.increase brightness and contrast
 15.black to alpha
 16.new drawable (ADDITION, 65%)
 17.fill it with black
 18.apply HSV noise
 19.apply a threshold
 20.apply a motion blur (length = 35, angle = 83)
 21.increase brightness and contrast
 22.black to alpha
 23.merge (in 2 steps) the 3 additional drawables
 24.if flatten, merge the rain-layers merged at previous steps into the BG
 25.rescale back to the original dimensions

 
 ----------------------------------------------------------------
 COPYRIGHT NOTICE
 ----------------------------------------------------------------
 
 This program is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation either version 3 of the License, or
 (at your option) any later version.  
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program if not, you can view the GNU General Public
 License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html
 Alternatively you can write to the Free Software Foundation, Inc., 675 Mass
 Ave, Cambridge, MA 02139, USA.
 
 -------------------------------------------------------------------
 other info
 -------------------------------------------------------------------
 the following line terminates the long initial comments
'''
#!/usr/bin/env python
from gimpfu import *
def RenderRain (inImage, inDrawable, doFlatten) :

    pdb.gimp_context_push
    currFG = pdb.gimp_context_get_foreground()
    currBG = pdb.gimp_context_get_background()
    
    owidth = inImage.width
    oheight = inImage.height
    type = inImage.base_type
    
    #gimp.message ("original dimensions = "+str(owidth)+" * "+str(oheight))

    # Set up an undo group, so the operation will be undone in one step.
    pdb.gimp_image_undo_group_start(inImage)

    # to get the best rain distribution, the original image is resized
    # scale :
    #        rescaling smallest dimension to 1000
    #        rescaling largest dimension proportionally
    if owidth <= oheight :
        pdb.gimp_image_scale (inImage, 1000, (oheight*1000/owidth))
    else :
        pdb.gimp_image_scale (inImage, (owidth*1000/oheight), 1000)
    rheight = inImage.height
    rwidth = inImage.width

    #gimp.message ("rescaled dimensions = "+str(rwidth)+" * "+str(rheight))
    
    pdb.gimp_context_set_foreground ((0, 0, 0, 255))

    rainLayer1 = pdb.gimp_layer_new_from_drawable (inDrawable, inImage)
    rainLayer1.name = "First Rain Layer"
    inImage.add_layer (rainLayer1, 0)
    pdb.gimp_layer_add_alpha (rainLayer1)
    rainLayer1.opacity = 95.00
    rainLayer1.mode = ADDITION_MODE 
    pdb.gimp_drawable_fill (rainLayer1, FOREGROUND_FILL)
    pdb.plug_in_hsv_noise (inImage, rainLayer1, 6, 180, 255, 255)
    pdb.gimp_threshold (rainLayer1, 127, 255)
    pdb.plug_in_mblur (inImage, rainLayer1, 0, 15, 75, 0, 0)
    pdb.gimp_brightness_contrast (rainLayer1, 115, 115)
    pdb.plug_in_colortoalpha (inImage, rainLayer1, ((0, 0, 0, 255)))

    rainLayer2 = pdb.gimp_layer_new_from_drawable (inDrawable, inImage)
    rainLayer2.name = "Second Rain Layer"
    inImage.add_layer (rainLayer2, 0)
    pdb.gimp_layer_add_alpha (rainLayer2)
    rainLayer2.opacity = 80.00
    rainLayer2.mode = ADDITION_MODE
    pdb.gimp_drawable_fill (rainLayer2, FOREGROUND_FILL)
    pdb.plug_in_hsv_noise (inImage, rainLayer2, 6, 180, 255, 255)
    pdb.gimp_threshold (rainLayer2, 127, 255)
    pdb.plug_in_mblur (inImage, rainLayer2, 0, 25, 79, 0, 0)
    pdb.gimp_brightness_contrast (rainLayer2, 118, 118)
    pdb.plug_in_colortoalpha (inImage, rainLayer2, ((0, 0, 0, 255)))

    rainLayer3 = pdb.gimp_layer_new_from_drawable (inDrawable, inImage)
    rainLayer3.name = "Third Rain Layer"
    inImage.add_layer (rainLayer3, 0)
    pdb.gimp_layer_add_alpha (rainLayer3)
    rainLayer3.opacity = 65.00
    rainLayer3.mode = ADDITION_MODE
    pdb.gimp_drawable_fill (rainLayer3, FOREGROUND_FILL)
    pdb.plug_in_hsv_noise (inImage, rainLayer3, 6, 180, 255, 255)
    pdb.gimp_threshold (rainLayer3, 127, 255)
    pdb.plug_in_mblur (inImage, rainLayer3, 0, 35, 83, 0, 0)
    pdb.gimp_brightness_contrast (rainLayer3, 115, 120)
    pdb.plug_in_colortoalpha (inImage, rainLayer3, ((0, 0, 0, 255)))

    rainLayer2 = pdb.gimp_image_merge_down (inImage, rainLayer3, 0) 
    rainLayer1 = pdb.gimp_image_merge_down (inImage, rainLayer2, 0)
    rainLayer1.name = "Merged Rain Layer!"
    
    if (doFlatten == True) :
        flattenedLayer = pdb.gimp_image_merge_down (inImage, rainLayer1, 0)
        flattenedLayer.name = "Layer with Rain!"
    
    pdb.gimp_image_scale (inImage, owidth, oheight)
    
    pdb.gimp_displays_flush
   
    # Close the undo group.
    pdb.gimp_image_undo_group_end(inImage)

    pdb.gimp_context_pop
    pdb.gimp_context_set_foreground(currFG)
    pdb.gimp_context_set_background(currBG)
    
    if (doFlatten == True) :
        return (flattenedLayer)
    else :
        return (rainLayer1)

# This is the plugin registration function
register(
    "RenderRain",    
    "to render the rain effect on the input drawable.",   
    "This script renders the rain effect on the input drawable.",
    "Diego", 
    "Diego Nassetti ", 
    "August 2016",
    "RenderRain", 
    "RGB*", 
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "idrawable", "Input drawable", None),
      (PF_TOGGLE, "flatten", "Flatten Image?", True),
    ], 
    [
      (PF_DRAWABLE, "odrawable", "Output drawable", None),  
    ],
    RenderRain,
    menu="<Image>/Diego/Ancillary",
    )

main()
