'''

 Filter to render the snow effect on the input drawable
 ....................................................................
 
 Release 1.0   initial
 
 Flow implemented to get the final result:      ...detail the major steps
 
 1. to get the best snow distribution, the original image is resized
    scale :
           rescaling smallest dimension to 1000
           rescaling largest dimension proportionally
 2. new drawable (ADDITION, 99%)
 3. fill it with black
 4. apply HSV noise
 5. apply a threshold 127-255 and dilate
 6. apply a motion blur (length = 3, angle = 45)
 7. apply a gaussian blur of 3 pix  
 8. black to alpha
 9. new drawable (ADDITION, 99%)
 10.fill it with black
 11.apply HSV noise
 12.apply a threshold 143-255 and dilate 2 times
 13.apply a motion blur (length = 6, angle = 135)
 14.apply a gaussian blur of 7 pix and increase brightness/contrast
 15.black to alpha
 16.new drawable (ADDITION, 99%)
 17.fill it with black
 18.apply HSV noise
 19.apply a threshold 159-255 and dilate 3 times
 20.apply a motion blur (length = 9, angle = 90)
 21.apply a gaussian blur of 11 pix and increase brightness/contrast
 22.black to alpha
 23.merge (in 2 steps) the 3 additional drawables
 24.if flatten, merge the snow-layers merged at previous steps into the BG
 25.rescale back to the original dimensions

 
 ----------------------------------------------------------------
 COPYRIGHT NOTICE
 ----------------------------------------------------------------
 
 This program is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation either version 3 of the License, or
 (at your option) any later version.  
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program if not, you can view the GNU General Public
 License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html
 Alternatively you can write to the Free Software Foundation, Inc., 675 Mass
 Ave, Cambridge, MA 02139, USA.
 
 -------------------------------------------------------------------
 other info
 -------------------------------------------------------------------
 the following line terminates the long initial comments
'''
#!/usr/bin/env python
from gimpfu import *
def RenderSnow (inImage, inDrawable, doFlatten) :

    pdb.gimp_context_push
    currFG = pdb.gimp_context_get_foreground()
    currBG = pdb.gimp_context_get_background()
    
    owidth = inImage.width
    oheight = inImage.height
    type = inImage.base_type
    
    #gimp.message ("original dimensions = "+str(owidth)+" * "+str(oheight))

    # Set up an undo group, so the operation will be undone in one step.
    pdb.gimp_image_undo_group_start(inImage)

    # to get the best snow distribution, the original image is resized
    # scale :
    #        rescaling smallest dimension to 1000
    #        rescaling largest dimension proportionally
    if owidth <= oheight :
        pdb.gimp_image_scale (inImage, 1000, (oheight*1000/owidth))
    else :
        pdb.gimp_image_scale (inImage, (owidth*1000/oheight), 1000)
    rheight = inImage.height
    rwidth = inImage.width

    #gimp.message ("rescaled dimensions = "+str(rwidth)+" * "+str(rheight))
    
    pdb.gimp_context_set_foreground ((0, 0, 0, 255))

    snowLayer1 = pdb.gimp_layer_new_from_drawable (inDrawable, inImage)
    snowLayer1.name = "First Snow Layer"
    inImage.add_layer (snowLayer1, 0)
    pdb.gimp_layer_add_alpha (snowLayer1)
    snowLayer1.opacity = 99.00
    snowLayer1.mode = ADDITION_MODE 
    pdb.gimp_drawable_fill (snowLayer1, FOREGROUND_FILL)
    pdb.plug_in_hsv_noise (inImage, snowLayer1, 6, 180, 255, 255)
    pdb.gimp_threshold (snowLayer1, 127, 255)
    pdb.plug_in_dilate (inImage, snowLayer1, 0, 0, 1.0, 15, 0 ,255)
    pdb.plug_in_mblur (inImage, snowLayer1, 0, 3, 45, 0, 0)
    pdb.plug_in_gauss (inImage, snowLayer1, 3, 3, 0)
    pdb.plug_in_colortoalpha (inImage, snowLayer1, ((0, 0, 0, 255)))

    snowLayer2 = pdb.gimp_layer_new_from_drawable (inDrawable, inImage)
    snowLayer2.name = "Second Snow Layer"
    inImage.add_layer (snowLayer2, 0)
    pdb.gimp_layer_add_alpha (snowLayer2)
    snowLayer2.opacity = 80.00
    snowLayer2.mode = ADDITION_MODE
    pdb.gimp_drawable_fill (snowLayer2, FOREGROUND_FILL)
    pdb.plug_in_hsv_noise (inImage, snowLayer2, 6, 180, 255, 255)
    pdb.gimp_threshold (snowLayer2, 143, 255)
    pdb.plug_in_dilate (inImage, snowLayer2, 0, 0, 1.0, 15, 0 ,255)
    pdb.plug_in_dilate (inImage, snowLayer2, 0, 0, 1.0, 15, 0 ,255)
    pdb.plug_in_mblur (inImage, snowLayer2, 0, 6, 135, 0, 0)
    pdb.plug_in_gauss (inImage, snowLayer2, 7, 7, 0)
    pdb.gimp_brightness_contrast (snowLayer2, 20, 20)
    pdb.plug_in_colortoalpha (inImage, snowLayer2, ((0, 0, 0, 255)))

    snowLayer3 = pdb.gimp_layer_new_from_drawable (inDrawable, inImage)
    snowLayer3.name = "Third Snow Layer"
    inImage.add_layer (snowLayer3, 0)
    pdb.gimp_layer_add_alpha (snowLayer3)
    snowLayer3.opacity = 65.00
    snowLayer3.mode = ADDITION_MODE
    pdb.gimp_drawable_fill (snowLayer3, FOREGROUND_FILL)
    pdb.plug_in_hsv_noise (inImage, snowLayer3, 6, 180, 255, 255)
    pdb.gimp_threshold (snowLayer3, 159, 255)
    pdb.plug_in_dilate (inImage, snowLayer3, 0, 0, 1.0, 15, 0 ,255)
    pdb.plug_in_dilate (inImage, snowLayer3, 0, 0, 1.0, 15, 0 ,255)
    pdb.plug_in_dilate (inImage, snowLayer3, 0, 0, 1.0, 15, 0 ,255)
    pdb.plug_in_mblur (inImage, snowLayer3, 0, 9, 90, 0, 0)
    pdb.plug_in_gauss (inImage, snowLayer3, 11, 11, 0)
    pdb.gimp_brightness_contrast (snowLayer3, 20, 20)
    pdb.plug_in_colortoalpha (inImage, snowLayer3, ((0, 0, 0, 255)))

    snowLayer2 = pdb.gimp_image_merge_down (inImage, snowLayer3, 0) 
    snowLayer1 = pdb.gimp_image_merge_down (inImage, snowLayer2, 0)
    snowLayer1.name = "Merged Snow Layer!"
    
    if (doFlatten == True) :
        flattenedLayer = pdb.gimp_image_merge_down (inImage, snowLayer1, 0)
        flattenedLayer.name = "Layer with Snow!"
    
    pdb.gimp_image_scale (inImage, owidth, oheight)
    
    pdb.gimp_displays_flush
   
    # Close the undo group.
    pdb.gimp_image_undo_group_end(inImage)

    pdb.gimp_context_pop
    pdb.gimp_context_set_foreground(currFG)
    pdb.gimp_context_set_background(currBG)
    
    if (doFlatten == True) :
        return (flattenedLayer)
    else :
        return ()

# This is the plugin registration function
register(
    "RenderSnow",    
    "to render the snow effect on the input drawable.",   
    "This script renders the snow effect on the input drawable.",
    "Diego", 
    "Diego Nassetti ", 
    "August 2016",
    "RenderSnow", 
    "RGB*", 
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "idrawable", "Input drawable", None),
      (PF_TOGGLE, "flatten", "Flatten Image?", True),
    ], 
    [
      (PF_DRAWABLE, "odrawable", "Output drawable", None),  
    ],
    RenderSnow,
    menu="<Image>/Diego/Ancillary",
    )

main()
