'''

 Filter to implement the stretching of each colour in the INPUT IMAGE  
 ....................................................................
 
 Release 1.0   initial
 
 Flow implemented to get the final result:       
 
 1. Ensure the requested percentage of 0 values in Red, Green, Blue
    (default 1%)
 2. Ensure the requested percentage of 255 values in Red, Green, Blue
    (default 1%)
 3. Apply the Levels adjustmentys using min-max previously calculated
 
 ----------------------------------------------------------------
 COPYRIGHT NOTICE
 ----------------------------------------------------------------
 
 This program is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation either version 3 of the License, or
 (at your option) any later version.  
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program if not, you can view the GNU General Public
 License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html
 Alternatively you can write to the Free Software Foundation, Inc., 675 Mass
 Ave, Cambridge, MA 02139, USA.
 
 -------------------------------------------------------------------
 other info
 -------------------------------------------------------------------
 
'''
#!/usr/bin/env python
from gimpfu import *
#==================================================================================================
# MAIN MODULE
#-------------------------------------------------------------------------------------------------- 
def Stretch_RGB_colours (inImage, inDrawable, inMin, inMax) :

    pdb.gimp_context_push
    currFG = pdb.gimp_context_get_foreground()
    currBG = pdb.gimp_context_get_background()
    
    # Set up an undo group, so the operation will be undone in one step.
    pdb.gimp_image_undo_group_start(inImage)

    # ============================
    # LEVELS STRETCH FOR COLOURS
    # ============================

    # RED 

    # =========== 
    # ZERO QUOTE
    # ===========  
    
    fromV = 0
    toV = 1
    percent = 0
    while (percent < inMin) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (inDrawable, HISTOGRAM_RED, fromV, toV)
        percent = percentile * 100  
        toV += 1
    #gimp.message ("percent RED at value 0 = " + str(percent))
    fromMinRED = toV - 1

    # =========== 
    # 255 QUOTE
    # ===========  
    
    fromV = 254
    toV = 255
    percent = 0
    while (percent < inMax) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (inDrawable, HISTOGRAM_RED, fromV, toV)
        percent = percentile * 100  
        fromV -= 1
    #gimp.message ("percent RED at value 255 = " + str(percent))
    toMaxRED = fromV + 1
    
    #gimp.message ("Red From,To = "+str(fromMinRED)+","+str(toMaxRED))
    
    # GREEN 

    # =========== 
    # ZERO QUOTE
    # ===========  
    
    fromV = 0
    toV = 1
    percent = 0
    while (percent < inMin) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (inDrawable, HISTOGRAM_GREEN, fromV, toV)
        percent = percentile * 100  
        toV += 1
    #gimp.message ("percent GREEN at value 0 = " + str(percent))
    fromMinGREEN = toV - 1

    # =========== 
    # 255 QUOTE
    # ===========  
    
    fromV = 254
    toV = 255
    percent = 0
    while (percent < inMax) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (inDrawable, HISTOGRAM_GREEN, fromV, toV)
        percent = percentile * 100  
        fromV -= 1
    #gimp.message ("percent GREEN at value 255 = " + str(percent))
    toMaxGREEN = fromV + 1
   
    #gimp.message ("Green From,To = "+str(fromMinGREEN)+","+str(toMaxGREEN))
    
    # BLUE 

    # =========== 
    # ZERO QUOTE
    # ===========  
    
    fromV = 0
    toV = 1
    percent = 0
    while (percent < inMin) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (inDrawable, HISTOGRAM_BLUE, fromV, toV)
        percent = percentile * 100  
        toV += 1
    #gimp.message ("percent BLUE at value 0 = " + str(percent))
    fromMinBLUE = toV - 1

    # =========== 
    # 255 QUOTE
    # ===========  
    
    fromV = 254
    toV = 255
    percent = 0
    while (percent < inMax) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (inDrawable, HISTOGRAM_BLUE, fromV, toV)
        percent = percentile * 100  
        fromV -= 1
    #gimp.message ("percent BLUE at value 255 = " + str(percent))
    toMaxBLUE = fromV + 1
   
    #gimp.message ("Blue From,To = "+str(fromMinBLUE)+","+str(toMaxBLUE))
    
    # =============================
    # APPLY MODIFICATIONS TO LEVELS
    # =============================
    
    pdb.gimp_levels (inDrawable, 1, fromMinRED, toMaxRED, 1, 0, 255)
    pdb.gimp_levels (inDrawable, 2, fromMinGREEN, toMaxGREEN, 1, 0, 255)
    pdb.gimp_levels (inDrawable, 3, fromMinBLUE, toMaxBLUE, 1, 0, 255)
    
    # Close the undo group.
    
    pdb.gimp_image_undo_group_end(inImage)
    
    pdb.gimp_context_pop
    pdb.gimp_context_set_foreground(currFG)
    pdb.gimp_context_set_background(currBG)

    pdb.gimp_displays_flush
    
    return(inDrawable)

# This is the plugin registration function
register(
    "Stretch_RGB_colours",    
    "to stretch R,G,B values of an image",   
    "This script stretches R,G,B values ensuring the desired min pixels % of 0 and 255 values",
    "Diego", 
    "Diego Nassetti ", 
    "August 2016",
    "Stretch RGB colours", 
    "RGB*", 
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "drawable", "Input drawable", None),
	  (PF_ADJUSTMENT, "colmin", "Percentage zero values",  01, (00, 25, 01)), 		#default, (min, max, increment)
	  (PF_ADJUSTMENT, "colmax", "Percentage 255 values",  01, (00, 25, 01)), 	   	#default, (min, max, increment)

    ], 
    [
      (PF_DRAWABLE, "outdrawable", "Output drawable", None),
    ],
    Stretch_RGB_colours,
    menu="<Image>/Diego/Ancillary/Colours",
    )

main()
